package com.yizhi.custom.api.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.custom.api.param.ExperiencePoolParam;
import com.yizhi.custom.api.vo.domain.CodePackageManageVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * 代码包管理feign接口
 *
 * @author dingxiaowei
 * @date 2018-11-5
 */
@FeignClient(name = "customproject", contextId = "CodePackageManageClient")
public interface CodePackageManageClient {

    /**
     * 代码包列表
     *
     * @param experiencePoolParam
     * @return
     */
    @GetMapping("/manage/codePackageManage/list")
    Page<CodePackageManageVo> getCodePackageManageList(@RequestBody ExperiencePoolParam experiencePoolParam);

    /**
     * 单个代码包查看
     *
     * @param CodePackageId
     * @return
     */
    @GetMapping("/manage/codePackageManage/view")
    CodePackageManageVo viewCodePackageManage(@RequestParam("CodePackageId") Long CodePackageId);

    /**
     * 删除代码包
     *
     * @param CodePackageId
     * @return
     */
    @GetMapping("/manage/codePackageManage/delete")
    boolean deleteCodePackage(@RequestParam("CodePackageId") Long CodePackageId);

    /**
     * 复制访问路径
     *
     * @param CodePackageId
     * @return
     */
    @GetMapping("/manage/codePackageManage/getUrl")
    String getCodePackageUrl(@RequestParam("CodePackageId") Long CodePackageId);

    /**
     * 导出二维码
     *
     * @param url
     * @param CodePackageId
     * @return
     */
    @GetMapping("/manage/codePackageManage/getQRCode")
    String getQRCode(@RequestParam("url") String url, @RequestParam("CodePackageId") Long CodePackageId);

    /**
     * 代码包新增(修改)
     *
     * @param
     * @return
     */
    @GetMapping("/manage/codePackageManage/insert")
    CodePackageManageVo insertCodePackage(@RequestParam(value = "CodePackageId", required = false) Long CodePackageId,
                                        @RequestParam(value = "CodePackageName", required = false) String CodePackageName,
                                        @RequestParam(value = "url", required = false) String url);

}
