package com.yizhi.custom.api.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.custom.api.param.ExperiencePoolParam;
import com.yizhi.custom.api.vo.custom.ExperiencePoolVoAutorizeAndProjectVO;
import com.yizhi.custom.api.vo.domain.ExperiencePoolVo;
import com.yizhi.custom.api.vo.domain.ProjectActivityExperienceVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;
import java.util.Map;

/**
 * 经验池主体feign接口
 *
 * @author dingxiaowei
 * @date 2018-11-2
 */
@FeignClient(name = "customproject", contextId = "ExperiencePoolClient")
public interface ExperiencePoolClient {

    /**
     * 显示经验池列表
     *
     * @param param
     * @return
     */
    @GetMapping("/manage/experiencePool/list")
    Page<ExperiencePoolVo> getExperiencePoolList(@RequestBody ExperiencePoolParam param);

    /**
     * 查看单个经验池查看
     *
     * @param poolId
     * @return
     */
    @GetMapping("/manage/experiencePool/view")
    ExperiencePoolVoAutorizeAndProjectVO getExperiencePoolView(@RequestParam("poolId") Long poolId);

    /**
     * 删除经验池
     *
     * @param experiencePoolId
     * @return
     */
    @GetMapping("/manage/experiencePool/delete")
    boolean deleteExperiencePool(@RequestParam("poolId") Long experiencePoolId);

    /**
     * 导入线下活动外部经验
     *
     * @param projectActivityExperienceList
     * @return
     */
    @PostMapping("/manage/experiencePool/importActivityExperiences")
    boolean insertExperiencePoolList(@RequestBody List<ProjectActivityExperienceVo> projectActivityExperienceList);

    /**
     * 下载经验规则
     *
     * @param poolId
     * @return
     */
    @GetMapping("/manage/experiencePool/getActivityExperience")
    List<ProjectActivityExperienceVo> getActivityExperienceList(@RequestParam("poolId") Long poolId);

    /**
     * 经验池保存(新增,修改下一步)(插入经验池可见范围及关联项目并返回项目关联活动列表)
     *
     * @param experiencePoolAutorizeAndProjectVO
     * @return
     */
    @PostMapping("/manage/experiencePool/save")
    Map<String, Object> insertExperiencePoolAuthorizeAndProject(@RequestBody
                                                                        ExperiencePoolVoAutorizeAndProjectVO experiencePoolAutorizeAndProjectVO);


    /**
     * 经验池新增完成
     *
     * @param poolId
     * @return
     */
    @GetMapping("/manage/experiencePool/complete")
    boolean completeExperiencePool(@RequestParam("poolId") Long poolId);


}
