package com.yizhi.custom.api.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.custom.api.vo.custom.ExperiencePoolReportExportVo;
import com.yizhi.custom.api.vo.custom.ExperiencePoolReportVo;
import com.yizhi.core.application.context.RequestContext;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * @author Ding
 * @className ExperiencePoolReportClient
 * @description TODO
 * @date 2018/11/29
 **/

@FeignClient(name = "customproject", contextId = "ExperiencePoolReportClient")
public interface ExperiencePoolReportClient {

    /**
     * 经验池报表获取列表
     *
     * @param poolName
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/manage/poolReport/list")
    Page<ExperiencePoolReportVo> poolReportList(@RequestParam(value = "poolName", required = false) String poolName,
                                                @RequestParam("pageNo") Integer pageNo,
                                                @RequestParam("pageSize") Integer pageSize,
                                                @RequestBody RequestContext requestContext);

    /**
     * 经验池报表查看列表
     *
     * @param poolId
     * @param key
     * @param orgName
     * @param pageNo
     * @param pageSize
     * @param type
     * @param siteId
     * @param requestContext
     * @param startTime
     * @param endTime
     * @return
     */
    @GetMapping("/manage/poolReport/list/view")
    Page<ExperiencePoolReportExportVo> viewPoolReportList(@RequestParam("poolId") Long poolId,
                                                          @RequestParam(value = "key", required = false) String key,
                                                          @RequestParam(value = "orgName", required = false) String orgName,
                                                          @RequestParam("pageNo") Integer pageNo,
                                                          @RequestParam("pageSize") Integer pageSize,
                                                          @RequestParam(value = "type", required = false) Integer type,
                                                          @RequestParam(value = "startTime", required = false) String startTime,
                                                          @RequestParam(value = "endTime", required = false) String endTime,
                                                          @RequestParam(value = "siteId", required = false) Long siteId,
                                                          @RequestBody RequestContext requestContext);

    /**
     * 经验池报表查看个人经验明细
     *
     * @param poolId
     * @param accountId
     * @param state
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/manage/poolReport/list/view/account/details")
    Page<ExperiencePoolReportExportVo> viewAccountDetails(@RequestParam("poolId") Long poolId,
                                                          @RequestParam("accountId") Long accountId,
                                                          @RequestParam(value = "state", required = false) Integer state,
                                                          @RequestParam("pageNo") Integer pageNo,
                                                          @RequestParam("pageSize") Integer pageSize);

    /**
     * 经验池报表查看个人经验明细下载
     *
     * @param poolId
     * @param accountId
     * @param state
     * @return
     */
    @GetMapping("/manage/poolReport/list/view/account/details/export")
    String viewAccountDetailsExport(@RequestParam("poolId") Long poolId,
                                    @RequestParam("accountId") Long accountId,
                                    @RequestParam(value = "state", required = false) Integer state);

    /**
     * 经验池报表导出所有人经验变更明细
     *
     * @param poolId
     * @return
     */
    @GetMapping("/manage/poolReport/list/view/all/details/export")
    String viewAllDetailsExport(@RequestParam("poolId") Long poolId);

    /**
     * 经验池报表导出阶段经验变更明细
     *
     * @param poolId
     * @param startTime
     * @param endTime
     * @param orgName
     * @param accountName
     * @return
     */
    @GetMapping("/manage/poolReport/list/view/stage/details/export")
    String viewStageDetailsExport(@RequestParam("poolId") Long poolId,
                                  @RequestParam(value = "startTime", required = false) String startTime,
                                  @RequestParam(value = "endTime", required = false) String endTime,
                                  @RequestParam(value = "orgName", required = false) String orgName,
                                  @RequestParam(value = "accountName", required = false) String accountName);

    /**
     * 经验池报表导出阶段经验消费变更明细
     *
     * @param poolId
     * @param startTime
     * @param endTime
     * @param orgName
     * @param accountName
     * @return
     */
    @GetMapping("/manage/poolReport/list/view/stage/purchase/details/export")
    String viewStagePurchaseDetailsExport(@RequestParam("poolId") Long poolId,
                                          @RequestParam(value = "startTime", required = false) String startTime,
                                          @RequestParam(value = "endTime", required = false) String endTime,
                                          @RequestParam(value = "orgName", required = false) String orgName,
                                          @RequestParam(value = "accountName", required = false) String accountName);


}
