package com.yizhi.custom.api.feign;

import java.util.List;
import java.util.Map;

import com.yizhi.custom.api.vo.custom.*;
import com.yizhi.custom.api.vo.domain.StudentExperiencePoolVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;

import com.baomidou.mybatisplus.plugins.Page;

@FeignClient(name = "customproject", contextId = "MyExperienceClient")
public interface MyExperienceClient {

    /**
     * 个人经验统计
     *
     * @param param
     * @return
     */
    @GetMapping("/api/my/experience")
    MyExperienceLevelVo getMyExperiencePool(@RequestParam("poolId") Long poolId, @RequestParam("sort") String sort);

    /**
     * 经验排行榜
     *
     * @param poolId
     * @param sort
     * @param pageSize
     * @param pageNo
     * @return
     */
    @GetMapping("/api/my/experience/ranking")
    Page<MyExperienceRankingVo> getMyExperienceRankings(@RequestParam("poolId") Long poolId,
                                                        @RequestParam("sort") String sort, @RequestParam("pageSize") Integer pageSize,
                                                        @RequestParam("pageNo") Integer pageNo);

    /**
     * 个人经验明细
     *
     * @param poolId
     * @param pageSize
     * @param pageNo
     * @return
     */
    @GetMapping("/api/my/experience/dtl")
    Page<MyExperienceEventsVo> getMyExperienceDtl(@RequestParam("poolId") Long poolId,
                                                  @RequestParam("pageSize") Integer pageSize, @RequestParam("pageNo") Integer pageNo);

    /**
     * 个人经验等级分类
     *
     * @param poolId
     * @param pageSize
     * @param pageNo
     * @return
     */
    @GetMapping("/api/my/experience/classify")
    MyExperienceLevelVo getMyExperienceClassify(@RequestParam("poolId") Long poolId,
                                                @RequestParam("pageSize") Integer pageSize, @RequestParam("pageNo") Integer pageNo);

    /**
     * 经验等级
     *
     * @param poolId
     * @return
     */
    @GetMapping("/api/my/experience/level")
    List<MyExperienceLevelVo> getMyExperienceLevels(@RequestParam("poolId") Long poolId);

    /**
     * 我的经验池登录信息
     *
     * @param poolId
     * @return
     */
    @GetMapping("/api/my/experience/login")
    MyExperiencePoolVo getMyExperienceLogin(@RequestParam("poolId") Long poolId);

    /**
     * 存步数
     *
     * @param poolId
     * @param step
     * @return
     */
    @GetMapping("/api/my/experience/savestep")
    StudentExperiencePoolVo saveMyExperienceStep(@RequestParam("poolId") Long poolId, @RequestParam("step") Integer step);

    /**
     * 存座右铭
     *
     * @param poolId
     * @param motto
     * @return
     */
    @GetMapping("/api/my/experience/savemotto")
    StudentExperiencePoolVo saveMyExperienceMotto(@RequestParam("poolId") Long poolId, @RequestParam("motto") String motto);

    /**
     * 自定义项目按组织经验排行
     *
     * @param poolId
     * @param pageNo
     * @param pageSize
     * @param type
     * @return
     */
    @GetMapping("/api/my/experience/orgRanking")
    Page<ExperienceOrgRankingVO> getOrgRanking(@RequestParam("poolId") Long poolId,
                                               @RequestParam("pageNo") Integer pageNo,
                                               @RequestParam("pageSize") Integer pageSize,
                                               @RequestParam("type") Integer type);

    /**
     * 自定义项目获取用户组别,批次,备注信息
     *
     * @param accountId
     * @param poolId
     * @return
     */
    @GetMapping("/api/my/experience/getBatchGroup")
    Map<String, Object> getBatchGroup(@RequestParam("accountId") Long accountId,
                                      @RequestParam("poolId") Long poolId);
}
