package com.yizhi.custom.api.feign;

import com.yizhi.custom.api.vo.domain.ProjectActivityExperienceSetVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Map;

/**
 * 经验池项目关联活动经验规则feign接口
 *
 * @author dingxiaowei
 * @date 2018-11-5
 */
@FeignClient(name = "customproject", contextId = "ProjectActivityExperienceSetClient")
public interface ProjectActivityExperienceSetClient {

    /**
     * 查看设置活动经验值规则列表
     *
     * @param activityExperienceId
     * @return
     */
    @GetMapping("/manage/experiencePool/projectActivityExperienceSet/list")
    Map<String, Object> getProjectActivityExperienceSets(@RequestParam("activityExperienceId") Long activityExperienceId,
                                                         @RequestParam(value = "setType", required = false) Integer setType);

    /**
     * 查看单个设置活动经验值规则
     *
     * @param activityExperienceSetId
     * @return
     */
    @GetMapping("/manage/experiencePool/projectActivityExperienceSet/view")
    ProjectActivityExperienceSetVo viewProjectActivityExperienceSet(@RequestParam("activityExperienceSetId") Long activityExperienceSetId);

    /**
     * 经验池项目关联活动经验规则添加(修改)
     *
     * @param activityExperienceId
     * @param setType
     * @param minScore
     * @param maxScore
     * @param experience
     * @param times
     * @return
     */
    @GetMapping("/manage/experiencePool/projectActivityExperienceSet/insert")
    Map<String, Object> insertProjectActivityExperienceSet(@RequestParam(value = "activityExperienceSetId", required = false) Long activityExperienceSetId,
                                                           @RequestParam(value = "activityExperienceId", required = true) Long activityExperienceId,
                                                           @RequestParam(value = "setType", required = true) Integer setType,
                                                           @RequestParam(value = "minScore", required = true) Integer minScore,
                                                           @RequestParam(value = "experience", required = true) Long experience);

    @GetMapping("/manage/experiencePool/projectActivityExperienceSet/insertWithScore")
    Map<String, Object> insertProjectActivityExperienceSetWithScore(@RequestParam(value = "activityExperienceSetId", required = false) Long activityExperienceSetId,
                                                                    @RequestParam(value = "activityExperienceId", required = true) Long activityExperienceId,
                                                                    @RequestParam(value = "setType", required = true) Integer setType,
                                                                    @RequestParam(value = "times", required = false) Integer times);

    /**
     * @param activityExperienceId
     * @return
     */
    @GetMapping("/manage/experiencePool/projectActivityExperienceSet/delete")
    boolean deleteProjectActivityExperienceSet(@RequestParam("projectActivityExperienceSetId") Long projectActivityExperienceSetId);

    @GetMapping("/manage/experiencePool/projectActivityExperienceSets/delete")
    Map<String, Object> deleteProjectActivityExperienceSets(@RequestParam("activityExperienceId") Long activityExperienceId);
}
