package com.yizhi.application.asynchronousTask;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.domain.ExperiencePool;
import com.yizhi.application.service.IExperiencePoolService;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.custom.api.vo.custom.ExperiencePoolReportExportVo;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.system.application.system.remote.OrganizationClient;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Ding
 * @className ViewAccountDetailsExport
 * @description 经验池报表查看个人经验明细下载
 * @date 2019/9/5
 **/
@Component("viewAccountDetailsExport1")
public class ViewAccountDetailsExport extends AbstractDefaultTask<Map, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(ViewAccountDetailsExport.class);

    @Autowired
    private IExperiencePoolService poolService;

    @Autowired
    private OrganizationClient organizationClient;

    @Override
    protected Map execute(Map<String, Object> map) {


        //从map中取信息
        Long taskId = (Long) map.get("taskId");
        String serialNo = (String) map.get("serialNo");
        String taskName = (String) map.get("taskName");
        Long poolId = (Long) map.get("poolId");
        Long accountId = (Long) map.get("accountId");
        Integer state = (Integer) map.get("state");
        RequestContext res = (RequestContext) map.get("res");

        //开始异步任务
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,
                res.getAccountId(), new Date(), res.getSiteId(), res.getCompanyId());
        working(taskContext);

        Page<ExperiencePoolReportExportVo> page = null;
        List<ExperiencePoolReportExportVo> list = null;

        //获取导出数据
        try {

            page = poolService.viewAccountDetails(poolId, accountId, state, 1, Integer.MAX_VALUE);
            if (null != page) {
                list = page.getRecords();
            }

            if (CollectionUtils.isEmpty(list)) {
                success(taskContext, "未找到数据", "");
            }
        } catch (Exception e) {
            LOGGER.error("获取数据过程中出现错误", e);
            fail(taskContext, "获取数据过程中出现错误");
        }


        //生成excel,上传阿里云返回url
        try {

            //获取经验池名称
            ExperiencePool experiencePool = poolService.selectById(poolId);
            String poolName = experiencePool.getExperiencePoolName();
            String accountName = "";
            if (list != null && list.size()>0){
                accountName = list.get(0).getAccountName();
            }



            //创建excel文件
            XSSFWorkbook wb = new XSSFWorkbook();
            //创建报表页
            XSSFSheet sheet = wb.createSheet("个人经验值变更明细下载");
            //创建表头
            XSSFRow row = sheet.createRow(0);


            //设置格式(水平垂直居中,加粗,字体大小14号)
            XSSFCellStyle style = wb.createCellStyle();
            style.setAlignment(HorizontalAlignment.CENTER);
            XSSFFont font = wb.createFont();
            font.setBold(true);
            font.setFontHeightInPoints((short) 14);
            style.setFont(font);


            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 6);
            //表头合并单元格
            sheet.addMergedRegion(cra);
            XSSFCell cell = row.createCell(0);

            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

            //填写表头名
            cell.setCellValue("用户" + accountName + "个人经验值变更明细");

            //表头居中
            cell.setCellStyle(style);


            cra = new CellRangeAddress(1, 1, 0, 6);
            //第二行合并单元格合并单元格
            sheet.addMergedRegion(cra);
            row = sheet.createRow(1);
            row.createCell(0).setCellValue("经验池: " + poolName);

            row = sheet.createRow(2);
            //重新设置格式(只加粗)
            style = wb.createCellStyle();
            font = wb.createFont();
            font.setBold(true);
            style.setFont(font);

            cell = row.createCell(0);
            cell.setCellStyle(style);
            cell.setCellValue("用户名");

            cell = row.createCell(1);
            cell.setCellStyle(style);
            cell.setCellValue("姓名");

            cell = row.createCell(2);
            cell.setCellStyle(style);
            cell.setCellValue("所在部门");

            cell = row.createCell(3);
            cell.setCellStyle(style);
            cell.setCellValue("变动值");

            cell = row.createCell(4);
            cell.setCellStyle(style);
            cell.setCellValue("时间");

            cell = row.createCell(5);
            cell.setCellStyle(style);
            cell.setCellValue("剩余经验值");

            cell = row.createCell(6);
            cell.setCellStyle(style);
            cell.setCellValue("明细");

            cell = row.createCell(7);
            cell.setCellStyle(style);
            cell.setCellValue("所在组织架构");


            Map<Long, String> orgNamesMap = new HashMap<>(16);
            //组装数据
            if (CollectionUtils.isNotEmpty(list)) {
                for (int i = 0; i < list.size(); i++) {
                    ExperiencePoolReportExportVo listVO = list.get(i);

                    row = sheet.createRow(i + 3);
                    row.createCell(0).setCellValue(listVO.getAccountName());
                    row.createCell(1).setCellValue(listVO.getAccountFullName());
                    row.createCell(2).setCellValue(listVO.getOrgName());
                    row.createCell(3).setCellValue(listVO.getExperience());
                    row.createCell(5).setCellValue(listVO.getExperienceTotal() == null ? "--" : listVO.getExperienceTotal().toString());

                    SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy/M/dd HH:mm:ss");

                    row.createCell(4).setCellValue(sdf1.format(listVO.getCreateTime()));

                    if (listVO.getType() == 10) {
                        row.createCell(6).setCellValue("兑换" + listVO.getActivityName() + "消耗" + (0 - listVO.getExperience()) + "经验值");
                    } else {
                        row.createCell(6).setCellValue(listVO.getDescription() == null ? "--" : listVO.getDescription());
                    }

                    if (null != orgNamesMap.get(listVO.getOrgId())) {
                        row.createCell(7).setCellValue(orgNamesMap.get(listVO.getOrgId()));
                    } else {
                        String orgNames = "";
                        List<String> orgNamesList = organizationClient.getChiefOrgNames(listVO.getOrgId(), res.getCompanyId());
                        if (CollectionUtils.isNotEmpty(orgNamesList)) {
                            for (String s : orgNamesList) {
                                if (StringUtils.isNotBlank(orgNames)) {
                                    orgNames = orgNames + "/" + s;
                                } else {
                                    orgNames = orgNames + s;
                                }
                            }
                            row.createCell(7).setCellValue(orgNames);
                            orgNamesMap.put(listVO.getOrgId(), orgNames);
                        }else {
                            row.createCell(7).setCellValue("--");
                            orgNamesMap.put(listVO.getOrgId(), "--");
                        }
                    }
                }
            }

            String fileName = accountName + "个人经验值变更明细" + sdf.format(new Date()) + ".xlsx";
//            String requestPath = "/Users/dingxiaowei/aaa";
            String requestPath = FileConstant.SAVE_PATH;
            File fileDir = new File(requestPath);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }

            String path = requestPath + "/" + fileName;
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                wb.write(os);
                //阿里云返回url
                String uploadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功导出" + list.size() + "条记录", uploadUrl);
            } catch (Exception e2) {
                e2.printStackTrace();
                fail(taskContext, "个人经验值变更明细下载写入过程中发生错误");
                LOGGER.error("个人经验值变更明细下载写入过程中发生错误");
            } finally {
                if (os != null) {
                    os.close();
                }
                if (wb != null) {
                    wb.close();
                }
                if (file != null) {
                    file.delete();
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, "个人经验值变更明细下载发生错误");
            LOGGER.error("个人经验值变更明细下载发生错误，请查看日志");
        }


        return null;
    }

}
