package com.yizhi.application.asynchronousTask;

import com.yizhi.application.domain.ExperiencePool;
import com.yizhi.application.service.IExperiencePoolService;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.custom.api.vo.custom.ExperiencePoolReportExportVo;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import org.apache.commons.collections.CollectionUtils;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author Ding
 * @className ViewAllDetailsExport
 * @description 经验池报表导出所有人经验变更明细
 * @date 2019/9/5
 **/
@Component
public class ViewAllDetailsExport extends AbstractDefaultTask<Map, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(ViewAllDetailsExport.class);

    @Autowired
    private IExperiencePoolService poolService;

    @Override
    protected Map execute(Map<String, Object> map) {


        //从map中取信息
        Long taskId = (Long) map.get("taskId");
        String serialNo = (String) map.get("serialNo");
        String taskName = (String) map.get("taskName");
        Long poolId = (Long) map.get("poolId");
        RequestContext res = (RequestContext) map.get("res");

        //开始异步任务
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,
                res.getAccountId(), new Date(), res.getSiteId(), res.getCompanyId());
        working(taskContext);

        List<ExperiencePoolReportExportVo> list = null;

        //获取导出数据
        try {

            list = poolService.viewAllDetails(poolId, null, null, null, null, null, res);

            if (CollectionUtils.isEmpty(list)) {
                success(taskContext, "未找到数据", "");
            }
        } catch (Exception e) {
            LOGGER.error("获取数据过程中出现错误", e);
            fail(taskContext, "获取数据过程中出现错误");
        }


        //生成excel,上传阿里云返回url
        try {

            //获取经验池名称
            ExperiencePool experiencePool = poolService.selectById(poolId);
            String poolName = experiencePool.getExperiencePoolName();


            //创建excel文件
            XSSFWorkbook wb = new XSSFWorkbook();
            //创建报表页
            XSSFSheet sheet = wb.createSheet("所有人经验值变更明细下载");
            //创建表头
            XSSFRow row = sheet.createRow(0);


            //设置格式(水平垂直居中,加粗,字体大小14号)
            XSSFCellStyle style = wb.createCellStyle();
            style.setAlignment(HorizontalAlignment.CENTER);
            XSSFFont font = wb.createFont();
            font.setBold(true);
            font.setFontHeightInPoints((short) 14);
            style.setFont(font);


            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 5);
            //表头合并单元格
            sheet.addMergedRegion(cra);
            XSSFCell cell = row.createCell(0);

            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

            //填写表头名
            cell.setCellValue("经验值变更明细统计");

            //表头居中
            cell.setCellStyle(style);


            cra = new CellRangeAddress(1, 1, 0, 5);
            //第二行合并单元格合并单元格
            sheet.addMergedRegion(cra);
            row = sheet.createRow(1);
            row.createCell(0).setCellValue("经验池: " + poolName);

            row = sheet.createRow(2);
            //重新设置格式(只加粗)
            style = wb.createCellStyle();
            font = wb.createFont();
            font.setBold(true);
            style.setFont(font);

            cell = row.createCell(0);
            cell.setCellStyle(style);
            cell.setCellValue("用户名");

            cell = row.createCell(1);
            cell.setCellStyle(style);
            cell.setCellValue("姓名");

            cell = row.createCell(2);
            cell.setCellStyle(style);
            cell.setCellValue("所在部门");

            cell = row.createCell(3);
            cell.setCellStyle(style);
            cell.setCellValue("变动值");

            cell = row.createCell(4);
            cell.setCellStyle(style);
            cell.setCellValue("时间");

            cell = row.createCell(5);
            cell.setCellStyle(style);
            cell.setCellValue("明细");

            cell = row.createCell(6);
            cell.setCellStyle(style);
            cell.setCellValue("所在组织架构");


            //组装数据
            if (CollectionUtils.isNotEmpty(list)) {
                for (int i = 0; i < list.size(); i++) {
                    ExperiencePoolReportExportVo listVO = list.get(i);

                    row = sheet.createRow(i + 3);
                    row.createCell(0).setCellValue(listVO.getAccountName());
                    row.createCell(1).setCellValue(listVO.getAccountFullName()==null?"--":listVO.getAccountFullName());
                    row.createCell(2).setCellValue(listVO.getOrgName()==null?"--":listVO.getOrgName());
                    row.createCell(3).setCellValue(listVO.getExperience());

                    SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy/M/dd HH:mm:ss");

                    row.createCell(4).setCellValue(sdf1.format(listVO.getCreateTime()));

                    row.createCell(5).setCellValue(listVO.getDescription() == null ? "--" : listVO.getDescription());

                    row.createCell(6).setCellValue(listVO.getOrgNames());
                }
            }

            String fileName = "所有人经验值变更明细下载" + sdf.format(new Date()) + ".xlsx";
//            String requestPath = "/Users/dingxiaowei/aaa";
            String requestPath = FileConstant.SAVE_PATH;
            File fileDir = new File(requestPath);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }

            String path = requestPath + "/" + fileName;
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                wb.write(os);
                //阿里云返回url
                String uploadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功导出" + list.size() + "条记录", uploadUrl);
            } catch (Exception e2) {
                e2.printStackTrace();
                fail(taskContext, "所有人经验值变更明细下载写入过程中发生错误");
                LOGGER.error("所有人经验值变更明细下载写入过程中发生错误");
            } finally {
                if (os != null) {
                    os.close();
                }
                if (wb != null) {
                    wb.close();
                }
                if (file != null) {
                    file.delete();
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, "所有人经验值变更明细下载发生错误");
            LOGGER.error("所有人经验值变更明细下载发生错误，请查看日志");
        }


        return null;
    }

}
