package com.yizhi.application.controller;


import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.domain.CodePackageManage;
import com.yizhi.application.service.ICodePackageManageService;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.custom.api.feign.CodePackageManageClient;
import com.yizhi.custom.api.param.ExperiencePoolParam;
import com.yizhi.custom.api.vo.domain.CodePackageManageVo;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.util.application.zip.ZipUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.*;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author ding
 * @since 2018-11-06
 */
@Component
@ConfigurationProperties(prefix = "bootstrap")
@RestController
@RequestMapping("/manage/codePackageManage")
public class CodePackageManageController implements CodePackageManageClient {

    private static final Logger LOGGER = LoggerFactory.getLogger(CodePackageManageController.class);

    @Value("${ACTIVE}")
    private String active;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ICodePackageManageService codePackageManageService;

    @Override
    @GetMapping("/list")
    public Page<CodePackageManageVo> getCodePackageManageList(@RequestBody ExperiencePoolParam experiencePoolParam) {
        Page<CodePackageManage> page = codePackageManageService.getCodePackageManageList(experiencePoolParam);
        Page<CodePackageManageVo> pageVo=new Page<>(page.getCurrent(),page.getSize());
        List<CodePackageManage> list=page.getRecords();
        List<CodePackageManageVo> lists=new ArrayList<>();
        if (!list.isEmpty()){
            for (CodePackageManage c:list) {
                CodePackageManageVo cc=new CodePackageManageVo();
                BeanUtils.copyProperties(c,cc);
                lists.add(cc);
            }
        }
        pageVo.setRecords(lists);
        return pageVo;
    }

    @Override
    @GetMapping("/view")
    public CodePackageManageVo viewCodePackageManage(Long CodePackageId) {
        CodePackageManage c=codePackageManageService.selectById(CodePackageId);
        CodePackageManageVo cv=new CodePackageManageVo();
        BeanUtils.copyProperties(c,cv);
        return cv;
    }

    @Override
    @GetMapping("/delete")
    public boolean deleteCodePackage(Long CodePackageId) {
        return codePackageManageService.deleteById(CodePackageId);
    }

    @Override
    @GetMapping("/getUrl")
    public String getCodePackageUrl(Long CodePackageId) {
        String url = null;
        CodePackageManage codePackageManage = codePackageManageService.selectById(CodePackageId);
        if (null != codePackageManage) {
            url = codePackageManage.getUrl();
        }
        return url;
    }

    @Override
    public String getQRCode(String url, Long CodePackageId) {
        return null;
    }

    @Override
    @GetMapping("/insert")
    public CodePackageManageVo insertCodePackage(@RequestParam(value = "CodePackageId", required = false) Long CodePackageId,
                                                 @RequestParam(value = "CodePackageName", required = false) String CodePackageName,
                                                 @RequestParam(value = "url", required = false) String url) {
        CodePackageManage codePackageManage = null;
        URL resultUrl = null;
        String viewUrl = null;
        HttpURLConnection connection = null;
        InputStream inputStream = null;
        long id = 0;
        LOGGER.info("###########################ACTIVE:" + active);
        try {
            //解析url
            String fileName = url.substring(url.lastIndexOf("/") + 1);
            String ossUrlPrefix = url.substring(0, url.lastIndexOf("/") + 1);
            String name = fileName.substring(0, fileName.lastIndexOf("."));
            url = ossUrlPrefix + URLEncoder.encode(fileName, "utf-8");

            resultUrl = new URL(url);
            connection = (HttpURLConnection) resultUrl.openConnection();
            //防止屏蔽程序抓取而返回403错误
            connection.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt)");
            inputStream = connection.getInputStream();

            String zipName = (CodePackageName + "代码包下载.zip");
            //用id命名对应代码包的指定文件夹
            if (null != CodePackageId) {
                id = CodePackageId;
            } else {
                id = idGenerator.generate();
            }
//            String requestPath = "/Users/dingxiaowei/aaa/" + id;
            String requestPath = "/home/" + id;
//            String savePath = "/Users/dingxiaowei/" + id;
            String savePath = "/CodePackage/" + id;
            viewUrl = "http://customproject.kmelearning.com/" + id + "/index.html";
            File fileDir = new File(requestPath);
            File saveFileDir = new File(savePath);
            if (saveFileDir.exists()) {
                deleteDir(saveFileDir);


            }
            String path = requestPath + "/" + zipName;
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }
            if (!saveFileDir.exists()) {
                saveFileDir.mkdir();
            }

            BufferedInputStream in = null;
            BufferedOutputStream out = null;
            try {
                in = new BufferedInputStream(inputStream);
                out = new BufferedOutputStream(new FileOutputStream(path));
                int len = -1;
                byte[] b = new byte[1024];
                while ((len = in.read(b)) != -1) {
                    out.write(b, 0, len);
                }
                in.close();
                out.close();
                //解压包
                ZipUtil.unZip(path, requestPath);
                //复制之前删除压缩包
                File file = new File(path);
                file.delete();
                //将解压文件复制到代码包文件夹(由于代码包文件夹结构只支持单个文件操作,所以在其他包解压),
                copyDir(requestPath, savePath);
                //删除临时文件夹
                boolean success = deleteDir(fileDir);
                LOGGER.info("##########################deleteSuccess:" + success);
                LOGGER.info("--------------------requestPath:" + requestPath);
            } catch (Exception e1) {
                LOGGER.error("解压代码包出错", e1);
            } finally {
                if (null != in) {
                    in.close();
                }
                if (null != out) {
                    out.close();
                }
            }

        } catch (MalformedURLException e) {
            LOGGER.error("创建ossurl 连接错误", e);
        } catch (IOException e) {
            LOGGER.error("创建ossurl 连接错误", e);
        }

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-ddHH:mm:ss");
        Date now = new Date();
        RequestContext res = ContextHolder.get();
        if (null != CodePackageId) {
            codePackageManage = codePackageManageService.selectById(CodePackageId);
            codePackageManage.setCodePackageName(CodePackageName);
            codePackageManage.setImportTime(now);
            codePackageManage.setUrl(viewUrl);
            codePackageManage.setUpdateById(res.getAccountId());
            codePackageManage.setUpdateByName(res.getAccountFullName());
            codePackageManage.setUpdateTime(now);
            codePackageManageService.updateById(codePackageManage);
        } else {
            codePackageManage = new CodePackageManage();
            codePackageManage.setId(id);
            codePackageManage.setCodePackageName(CodePackageName);
            codePackageManage.setImportTime(now);
            codePackageManage.setUrl(viewUrl);
            codePackageManage.setCreateById(res.getAccountId());
            codePackageManage.setCreateByName(res.getAccountFullName());
            codePackageManage.setCreateTime(now);
            codePackageManage.setCompanyId(res.getCompanyId());
            codePackageManage.setSiteId(res.getSiteId());
            codePackageManage.setOrgId(res.getOrgId());
            codePackageManageService.insert(codePackageManage);
        }
        CodePackageManageVo cv=new CodePackageManageVo();
        BeanUtils.copyProperties(codePackageManage,cv);
        return cv;
    }

    //复制文件夹方法
    public static void copyDir(String sourcePath, String newPath) throws IOException {
        File file = new File(sourcePath);
        String[] filePath = file.list();

        if (!(new File(newPath)).exists()) {
            (new File(newPath)).mkdir();
        }

        for (int i = 0; i < filePath.length; i++) {
            if ((new File(sourcePath + file.separator + filePath[i])).isDirectory()) {
                copyDir(sourcePath + file.separator + filePath[i], newPath + file.separator + filePath[i]);
            }

            if (new File(sourcePath + file.separator + filePath[i]).isFile()) {
                copyFile(sourcePath + file.separator + filePath[i], newPath + file.separator + filePath[i]);
            }

        }
    }

    //复制文件方法
    public static void copyFile(String oldPath, String newPath) throws IOException {
        File oldFile = new File(oldPath);
        File file = new File(newPath);
        FileInputStream in = new FileInputStream(oldFile);
        FileOutputStream out = new FileOutputStream(file);
        ;

        byte[] buffer = new byte[2097152];
        int readByte = 0;
        while ((readByte = in.read(buffer)) != -1) {
            out.write(buffer, 0, readByte);
        }

        in.close();
        out.close();
    }


    //删除法目录方法
    private static boolean deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            //递归删除目录中的子目录下
            for (int i = 0; i < children.length; i++) {
                boolean success = deleteDir(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }
        // 目录此时为空，可以删除
        return dir.delete();
    }

}

