package com.yizhi.application.controller;


import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.domain.ExperiencePoolLevel;
import com.yizhi.application.service.IExperiencePoolLevelService;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.custom.api.feign.ExperiencePoolLevelClient;
import com.yizhi.custom.api.vo.domain.ExperiencePoolLevelVo;
import com.yizhi.application.orm.id.IdGenerator;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author ding
 * @since 2018-11-06
 */
@RestController
@RequestMapping("/manage/experiencePool")
public class ExperiencePoolLevelController implements ExperiencePoolLevelClient {

    @Autowired
    IdGenerator idGenerator;

    @Autowired
    private IExperiencePoolLevelService poolLevelService;

    @Override
    @GetMapping("/level/list")
    public List<ExperiencePoolLevelVo> getExperiencePoolLevelList(Long poolId) {
        ExperiencePoolLevel poolLevel = new ExperiencePoolLevel();
        poolLevel.setExperiencePoolId(poolId);
        EntityWrapper<ExperiencePoolLevel> entityWrapper = new EntityWrapper<>(poolLevel);
        entityWrapper.orderBy("min_level_experience", true);
        List<ExperiencePoolLevel> list = poolLevelService.selectList(entityWrapper);
        List<ExperiencePoolLevelVo> levelVos=new ArrayList<>();
        if (!list.isEmpty()) {
            for (ExperiencePoolLevel pl : list) {
                ExperiencePoolLevelVo pls=new ExperiencePoolLevelVo();
                BeanUtils.copyProperties(pl,pls);
                levelVos.add(pls);
            }
        }
        return levelVos;
    }

    @Override
    @GetMapping("/level/view")
    public ExperiencePoolLevelVo getExperiencePoolLevelView(Long poolId) {
        ExperiencePoolLevel pl=poolLevelService.selectById(poolId);
        ExperiencePoolLevelVo pls=new ExperiencePoolLevelVo();
        BeanUtils.copyProperties(pl,pls);
        return pls;
    }

    @Override
    @GetMapping("/level/delete")
    public boolean deleteExperiencePoolLevel(Long poolLevelId) {

        return poolLevelService.deleteById(poolLevelId);
    }



    @Override
    @GetMapping("/level/insert")
    public Map<String, Object> insertExperiencePoolLevel(@RequestBody ExperiencePoolLevelVo experiencePoolLevel) {
        Map<String, Object> resultMap = new HashMap<>();
        RequestContext res = ContextHolder.get();
        ExperiencePoolLevel experiencePoolLevel1 = new ExperiencePoolLevel();
        experiencePoolLevel1.setLevelNo(experiencePoolLevel.getLevelNo());
        experiencePoolLevel1.setExperiencePoolId(experiencePoolLevel.getExperiencePoolId());
        EntityWrapper<ExperiencePoolLevel> entityWrapper = new EntityWrapper<>(experiencePoolLevel1);
        //如果是修改,重复编码和等级验证
        if (null != experiencePoolLevel.getId()) {
            if (!poolLevelService.selectById(experiencePoolLevel.getId()).getLevelNo().equals(experiencePoolLevel.getLevelNo())
                    && CollectionUtils.isNotEmpty(poolLevelService.selectList(entityWrapper))) {
                resultMap.put("isOk", false);
                return resultMap;
            } else {
                experiencePoolLevel1 = new ExperiencePoolLevel();
                experiencePoolLevel1.setMinLevelExperience(experiencePoolLevel.getMinLevelExperience());
                experiencePoolLevel1.setExperiencePoolId(experiencePoolLevel.getExperiencePoolId());
                entityWrapper = new EntityWrapper<>(experiencePoolLevel1);
                if (!poolLevelService.selectById(experiencePoolLevel.getId()).getMinLevelExperience().equals(experiencePoolLevel.getMinLevelExperience())
                        && CollectionUtils.isNotEmpty(poolLevelService.selectList(entityWrapper))) {
                    resultMap.put("isOk", false);
                    return resultMap;
                } else {
                    resultMap.put("isOk", true);
                }
            }
        } else {
            //如果是新建,重复编码和等级验证
            if (CollectionUtils.isNotEmpty(poolLevelService.selectList(entityWrapper))) {
                resultMap.put("isOk", false);
                return resultMap;
            } else {
                experiencePoolLevel1 = new ExperiencePoolLevel();
                experiencePoolLevel1.setMinLevelExperience(experiencePoolLevel.getMinLevelExperience());
                experiencePoolLevel1.setExperiencePoolId(experiencePoolLevel.getExperiencePoolId());
                entityWrapper = new EntityWrapper<>(experiencePoolLevel1);
                if (CollectionUtils.isNotEmpty(poolLevelService.selectList(entityWrapper))) {
                    resultMap.put("isOk", false);
                    return resultMap;
                } else {
                    resultMap.put("isOk", true);
                }
            }
        }
        ExperiencePoolLevel plv=new ExperiencePoolLevel();
        BeanUtils.copyProperties(experiencePoolLevel,plv);
        //修改更新入库
        if (null != plv.getId()) {
            plv.setUpdateById(res.getAccountId());
            plv.setUpdateByName(res.getAccountFullName());
            plv.setUpdateTime(new Date());
            poolLevelService.updateById(plv);
        } else {
            //新建入库
            plv.setId(idGenerator.generate());
            plv.setCreateTime(new Date());
            plv.setCreateById(res.getAccountId());
            plv.setCreateByName(res.getAccountFullName());
            poolLevelService.insert(plv);
        }
        resultMap.put("experiencePoolLevel", experiencePoolLevel);
        return resultMap;
    }

}

