package com.yizhi.application.controller;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.service.IExperiencePoolService;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.custom.api.feign.ExperiencePoolReportClient;
import com.yizhi.custom.api.vo.custom.ExperiencePoolReportExportVo;
import com.yizhi.custom.api.vo.custom.ExperiencePoolReportVo;
import com.yizhi.application.asynchronousTask.ViewAccountDetailsExport;
import com.yizhi.application.asynchronousTask.ViewAllDetailsExport;
import com.yizhi.application.asynchronousTask.ViewStageDetailsExport;
import com.yizhi.application.asynchronousTask.ViewStagePurchaseDetailsExport;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.application.orm.id.IdGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;

/**
 * @author Ding
 * @className ExperiencePoolReportController
 * @description TODO
 * @date 2018/11/29
 **/

@RestController
public class ExperiencePoolReportController implements ExperiencePoolReportClient {

    @Autowired
    private IExperiencePoolService poolService;

    @Autowired
    private ViewAccountDetailsExport viewAccountDetailsExport;

    @Autowired
    private ViewAllDetailsExport viewAllDetailsExport;

    @Autowired
    private ViewStageDetailsExport viewStageDetailsExport;

    @Autowired
    private ViewStagePurchaseDetailsExport viewStagePurchaseDetailsExport;

    @Autowired
    private IdGenerator idGenerator;

    @Override
    public Page<ExperiencePoolReportVo> poolReportList(@RequestParam(value = "poolName", required = false) String poolName,
                                                       @RequestParam("pageNo") Integer pageNo, @RequestParam("pageSize") Integer pageSize,
                                                       @RequestBody RequestContext requestContext) {
        return poolService.getPoolReportList(poolName, pageNo, pageSize, requestContext);
    }

    @Override
    public Page<ExperiencePoolReportExportVo> viewPoolReportList(Long poolId, String key, String orgName, Integer pageNo, Integer pageSize,
                                                                 Integer type, String startTime, String endTime, Long siteId,@RequestBody RequestContext requestContext) {
        return poolService.getReportViewList(poolId, key, orgName, pageNo, pageSize, type, startTime, endTime, siteId, requestContext);
    }


    @Override
    public Page<ExperiencePoolReportExportVo> viewAccountDetails(Long poolId, Long accountId, Integer state, Integer pageNo, Integer pageSize) {
        return poolService.viewAccountDetails(poolId, accountId, state, pageNo, pageSize);
    }

    @Override
    public String viewAccountDetailsExport(Long poolId, Long accountId, Integer state) {

        Map<String, Object> map = new HashMap<>();
        Long taskId = idGenerator.generate();
        String taskName = "经验-查看-按用户统计-查看-经验值变更明细下载-" + taskId.toString();
        String serialNo = "EXPERIENCE-VIEW-ACCOUNT-DETAILS-" + taskId.toString();
        RequestContext res = ContextHolder.get();
        map.put("taskId", taskId);
        map.put("taskName", taskName);
        map.put("serialNo", serialNo);
        map.put("res", res);
        map.put("poolId", poolId);
        map.put("accountId", accountId);
        map.put("state", state);
        viewAccountDetailsExport.execute(map, true);

        return serialNo;
    }

    @Override
    public String viewAllDetailsExport(Long poolId) {
        Map<String, Object> map = new HashMap<>();
        Long taskId = idGenerator.generate();
        String taskName = "经验-查看-导出全部变更明细-" + taskId.toString();
        String serialNo = "EXPERIENCE-VIEW-ALL-DETAILS-" + taskId.toString();
        RequestContext res = ContextHolder.get();
        map.put("taskId", taskId);
        map.put("taskName", taskName);
        map.put("serialNo", serialNo);
        map.put("res", res);
        map.put("poolId", poolId);
        viewAllDetailsExport.execute(map, true);

        return serialNo;
    }

    @Override
    public String viewStageDetailsExport(Long poolId, String startTime, String endTime, String orgName, String accountName) {
        Map<String, Object> map = new HashMap<>();
        Long taskId = idGenerator.generate();
        String taskName = "经验-查看-阶段经验值变动统计-明细导出-" + taskId.toString();
        String serialNo = "EXPERIENCE-VIEW-STAGE-DETAILS-" + taskId.toString();
        RequestContext res = ContextHolder.get();
        map.put("taskId", taskId);
        map.put("taskName", taskName);
        map.put("serialNo", serialNo);
        map.put("res", res);
        map.put("poolId", poolId);
        map.put("startTime", startTime);
        map.put("endTime", endTime);
        map.put("orgName", orgName);
        map.put("accountName", accountName);
        viewStageDetailsExport.execute(map, true);

        return serialNo;
    }

    @Override
    public String viewStagePurchaseDetailsExport(Long poolId, String startTime, String endTime, String orgName, String accountName) {
        Map<String, Object> map = new HashMap<>();
        Long taskId = idGenerator.generate();
        String taskName = "经验-查看-阶段经验值变动统计-消耗明细导出-" + taskId.toString();
        String serialNo = "EXPERIENCE-VIEW-STAGE-PURCHASE-DETAILS-" + taskId.toString();
        RequestContext res = ContextHolder.get();
        map.put("taskId", taskId);
        map.put("taskName", taskName);
        map.put("serialNo", serialNo);
        map.put("res", res);
        map.put("poolId", poolId);
        map.put("startTime", startTime);
        map.put("endTime", endTime);
        map.put("orgName", orgName);
        map.put("accountName", accountName);
        viewStagePurchaseDetailsExport.execute(map, true);

        return serialNo;
    }


}
