package com.yizhi.application.controller;


import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.domain.ExperiencePoolAuthorize;
import com.yizhi.application.domain.StudentExperiencePool;
import com.yizhi.application.mapper.ExperiencePoolAuthorizeMapper;
import com.yizhi.application.mapper.MyExperienceMapper;
import com.yizhi.application.service.IExperiencePoolAuthorizeService;
import com.yizhi.application.service.IStudentExperienceService;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.custom.api.feign.MyExperienceClient;
import com.yizhi.custom.api.vo.custom.*;
import com.yizhi.custom.api.vo.domain.StudentExperiencePoolVo;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.vo.AccountVO;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RestController;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 我的经验
 * </p>
 *
 * @author ding
 * @since 2018-11-06
 */
@RestController
public class MyExperienceController implements MyExperienceClient {

    @Autowired
    private MyExperienceMapper myExperienceMapper;

    @Autowired
    private ExperiencePoolAuthorizeMapper experiencePoolAuthorizeMapper;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private IExperiencePoolAuthorizeService experiencePoolAuthorizeService;

    @Autowired
    private AccountClient accountClient;

    @Autowired
    private IStudentExperienceService studentExperienceService;


    @Override
    public MyExperienceLevelVo getMyExperiencePool(Long poolId, String sort) {
        RequestContext res = ContextHolder.get();
        if (res == null) {
            return null;
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String currentTime = sdf.format(new Date());

        Long accountId = res.getAccountId();
        AccountVO accountVO = accountClient.findById(accountId);
        if (null == accountVO) {
            MyExperienceLevelVo myExperienceLevelVo = new MyExperienceLevelVo();
            myExperienceLevelVo.setCurrentTime(currentTime);
            return myExperienceLevelVo;
        }
        MyExperienceLevelVo myExperienceLevelVo = myExperienceMapper.getMyExperiencePoolAndLevel(poolId, accountId);
        if (myExperienceLevelVo == null) {
            myExperienceLevelVo = myExperienceMapper.getMyExperiencePool(poolId, accountId);

            Integer experienceTotal = myExperienceLevelVo.getExperienceTotal();

            if (null == experienceTotal) {
                myExperienceLevelVo.setRanking("0");
            } else {
                String s2 = "2";
                String s3 = "3";
                String group = null;
                String batchNo = null;
                if (null != sort && s2.equals(sort.trim())) {
                    group = getGroupOrBatchNo(poolId, res.getAccountId(), 2);
                }
                if (null != sort && s3.equals(sort.trim())) {
                    batchNo = getGroupOrBatchNo(poolId, res.getAccountId(), 3);
                }
                Integer ranking = myExperienceMapper.getMyExperienceRanking(poolId, experienceTotal, group, batchNo);
                if (ranking == null) {
                    ranking = 0;
                } else {
                    ranking += 1;
                }
                myExperienceLevelVo.setRanking(ranking.toString());
            }

            if (myExperienceLevelVo == null) {
                myExperienceLevelVo = new MyExperienceLevelVo();
                myExperienceLevelVo.setCurrentTime(currentTime);
                return myExperienceLevelVo;
            }
            myExperienceLevelVo.setName(accountVO.getName() == null ? "" : accountVO.getName());
            myExperienceLevelVo.setFullName(accountVO.getFullName() == null ? "" : accountVO.getFullName());
            myExperienceLevelVo.setSex(accountVO.getSex() == null ? "" : accountVO.getSex());
            myExperienceLevelVo.setCurrentTime(currentTime);
            return myExperienceLevelVo;
        }

        myExperienceLevelVo.setName(accountVO.getName() == null ? "" : accountVO.getName());
        myExperienceLevelVo.setFullName(accountVO.getFullName() == null ? "" : accountVO.getFullName());
        myExperienceLevelVo.setSex(accountVO.getSex() == null ? "" : accountVO.getSex());

        //获取个人排名
        Integer experienceTotal = myExperienceLevelVo.getExperienceTotal();
        String s2 = "2";
        String s3 = "3";
        String group = null;
        String batchNo = null;
        if (null != sort && s2.equals(sort.trim())) {
            group = getGroupOrBatchNo(poolId, res.getAccountId(), 2);
        }
        if (null != sort && s3.equals(sort.trim())) {
            batchNo = getGroupOrBatchNo(poolId, res.getAccountId(), 3);
        }
        Integer ranking = myExperienceMapper.getMyExperienceRanking(poolId, experienceTotal, group, batchNo);
        if (ranking == null) {
            ranking = 0;
        } else {
            ranking += ranking;
        }
        myExperienceLevelVo.setRanking(ranking.toString());
        myExperienceLevelVo.setCurrentTime(currentTime);

        return myExperienceLevelVo;
    }

    @Override
    public Page<MyExperienceRankingVo> getMyExperienceRankings(Long poolId, String sort, Integer pageSize,
                                                               Integer pageNo) {
        RequestContext res = ContextHolder.get();
        if (res == null) {
            return null;
        }
        String s2 = "2";
        String s3 = "3";
        String group = null;
        String batchNo = null;
        if (null != sort && s2.equals(sort.trim())) {
            group = getGroupOrBatchNo(poolId, res.getAccountId(), 2);
        }
        if (null != sort && s3.equals(sort.trim())) {
            batchNo = getGroupOrBatchNo(poolId, res.getAccountId(), 3);
        }
        List<MyExperienceRankingVo> rankings = myExperienceMapper.getMyExperienceRankings(poolId, group, batchNo, pageSize, pageSize * (pageNo - 1));
        if (rankings == null) {
            return null;
        }

        //获取个人排名
        for (MyExperienceRankingVo ranking : rankings) {
            if (ranking == null) {
                continue;
            }
            Integer total = myExperienceMapper.getMyExperienceRanking(poolId, ranking.getExperience(), group, batchNo);
            if (total == null) {
                total = 0;
            } else {
                total += 1;
            }
            ranking.setRanking(total);
        }

        Page<MyExperienceRankingVo> page = new Page<MyExperienceRankingVo>(pageNo, pageSize);
        if (CollectionUtils.isEmpty(rankings)) {
            page.setRecords(new ArrayList<MyExperienceRankingVo>());
        } else {
            page.setRecords(rankings);
        }
        Integer count = myExperienceMapper.getMyExperienceRankingsPageTotal(poolId, group, batchNo);
        if (count == null) {
            count = 0;
        }
        page.setTotal(count);
        return page;
    }

    @Override
    public Page<MyExperienceEventsVo> getMyExperienceDtl(Long poolId, Integer pageSize, Integer pageNo) {
        RequestContext res = ContextHolder.get();
        if (res == null) {
            return null;
        }

        Page<MyExperienceEventsVo> page = new Page<MyExperienceEventsVo>(pageNo, pageSize);
        List<MyExperienceEventsVo> evs = myExperienceMapper.getMyExperienceDtl(poolId, res.getAccountId(), page);
        if (evs == null) {
            return page;
        }
        page.setRecords(evs);

        return page;
    }

    @Override
    public MyExperienceLevelVo getMyExperienceClassify(Long poolId, Integer pageSize, Integer pageNo) {
        RequestContext res = ContextHolder.get();
        if (res == null) {
            return null;
        }

        Long accountId = res.getAccountId();
        AccountVO accountVO = accountClient.findById(accountId);
        if (null == accountVO) {
            return null;
        }
        MyExperienceLevelVo myExperienceLevelVo = myExperienceMapper.getMyExperiencePoolAndLevel(poolId, accountId);
        if (myExperienceLevelVo == null) {
            myExperienceLevelVo = myExperienceMapper.getMyExperiencePool(poolId, accountId);
            if (myExperienceLevelVo == null) {
                myExperienceLevelVo = new MyExperienceLevelVo();
                myExperienceLevelVo.setFullName(accountVO.getFullName());
                myExperienceLevelVo.setName(accountVO.getName());
                myExperienceLevelVo.setSex(accountVO.getSex());
                return myExperienceLevelVo;
            }
            myExperienceLevelVo.setLevelCode("");
            myExperienceLevelVo.setLevelName("");
            myExperienceLevelVo.setRanking("");
        }
        myExperienceLevelVo.setName(accountVO.getName());
        myExperienceLevelVo.setFullName(accountVO.getFullName());
        myExperienceLevelVo.setSex(accountVO.getSex());


        if (null != myExperienceLevelVo.getExperienceTotal()) {
            Integer total = myExperienceMapper.getMyExperienceRanking(poolId, myExperienceLevelVo.getExperienceTotal(), null, null);
            if (null == total) {
                total = 0;
            } else {
                total += 1;
            }
            myExperienceLevelVo.setRanking(total.toString());
        }

        //获取分类明细
        List<MyExperienceEventsVo> evs = myExperienceMapper.getMyExperience(poolId, res.getAccountId());
        if (CollectionUtils.isNotEmpty(evs)) {
            myExperienceLevelVo.setEventClassifys(evs);
        }

        return myExperienceLevelVo;
    }

    @Override
    public List<MyExperienceLevelVo> getMyExperienceLevels(Long poolId) {
        return myExperienceMapper.getMyExperienceLevels(poolId);
    }

    @Override
    public MyExperiencePoolVo getMyExperienceLogin(Long poolId) {
        RequestContext res = ContextHolder.get();
        if (res == null) {
            return null;
        }
        Long accountId = res.getAccountId();
        MyExperiencePoolVo pool = myExperienceMapper.getMyExperienceLogin(poolId, accountId);
        if (pool == null) {

            Date date = new Date();
            StudentExperiencePool studentExperiencePool = createNewStudentExperiencePool(poolId, accountId);
            if (studentExperiencePool == null) {
                return null;
            }
            studentExperiencePool.setCreateById(res.getAccountId());
            studentExperiencePool.setCreateByName(res.getAccountName());
            studentExperiencePool.setCreateTime(date);
            myExperienceMapper.insert(studentExperiencePool);

            pool = new MyExperiencePoolVo();
            pool.setFirstLogin(0);
            pool.setMotto("");
            pool.setStep(0);
            pool.setNowTime(date);
            pool.setAccountCode(studentExperiencePool.getBatchNo());
        } else {
            pool.setFirstLogin(1);
        }

        return pool;
    }

    @Override
    public StudentExperiencePoolVo saveMyExperienceStep(Long poolId, Integer step) {
        RequestContext res = ContextHolder.get();
        if (res == null) {
            return null;
        }
        Long accountId = res.getAccountId();
        Date date = new Date();

        StudentExperiencePool pool = myExperienceMapper.getMyExperiencePoolInfo(poolId, accountId);
        if (pool == null) {
            pool = createNewStudentExperiencePool(poolId, accountId);
            if (pool == null) {
                return null;
            }
            pool.setCreateByName(res.getAccountName());
            pool.setCreateTime(date);
            pool.setStep(step);
            myExperienceMapper.insert(pool);
        } else {
            pool.setUpdateById(accountId);
            pool.setUpdateByName(res.getAccountName());
            pool.setUpdateTime(date);
            pool.setStep(step);
            myExperienceMapper.updateAllColumnById(pool);
        }
        StudentExperiencePoolVo spv=new StudentExperiencePoolVo();
        BeanUtils.copyProperties(pool,spv);
        return spv;
    }

    @Override
    public StudentExperiencePoolVo saveMyExperienceMotto(Long poolId, String motto) {
        RequestContext res = ContextHolder.get();
        if (res == null) {
            return null;
        }
        Long accountId = res.getAccountId();
        Date date = new Date();

        StudentExperiencePool pool = myExperienceMapper.getMyExperiencePoolInfo(poolId, accountId);
        if (pool == null) {
            pool = createNewStudentExperiencePool(poolId, accountId);
            if (pool == null) {
                return null;
            }
            pool.setCreateByName(res.getAccountName());
            pool.setCreateTime(date);
            pool.setMotto(motto);
            myExperienceMapper.insert(pool);
        } else {
            pool.setUpdateById(accountId);
            pool.setUpdateByName(res.getAccountName());
            pool.setUpdateTime(date);
            pool.setMotto(motto);
            myExperienceMapper.updateAllColumnById(pool);
        }

        StudentExperiencePoolVo spv=new StudentExperiencePoolVo();
        BeanUtils.copyProperties(pool,spv);
        return spv;
    }

    @Override
    public Page<ExperienceOrgRankingVO> getOrgRanking(Long poolId, Integer pageNo, Integer pageSize, Integer type) {
        return studentExperienceService.getOrgRanking(poolId, pageNo, pageSize, type);
    }

    @Override
    public Map<String, Object> getBatchGroup(Long accountId, Long poolId) {
        return myExperienceMapper.getBatchGroup(accountId, poolId);
    }


    /**
     * 创建对象，设置：主键ID，批次号，登录序号
     *
     * @param poolId
     * @param accountId
     * @return
     */
    private StudentExperiencePool createNewStudentExperiencePool(Long poolId, Long accountId) {
        String key = poolId.toString();

//        //查询权限
//        ExperiencePoolAuthorizeVo auth = experiencePoolAuthorizeMapper.getExperiencePoolAuthorizeByPoolIdAndAccountId(poolId, accountId);
//        if (auth == null) {
//            return null;
//        }

        StudentExperiencePool pool = new StudentExperiencePool();
        pool.setId(idGenerator.generate());
        pool.setExperiencePoolId(poolId);
//        pool.setBatchNo(auth.getBatchNo());
        pool.setAccountId(accountId);
        pool.setCreateById(accountId);

        Integer num = myExperienceMapper.getMaxLoginCount(poolId);
        if (num == null) {
            num = 0;
        }
        pool.setNum(num + 1);

        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMM");
        String batchNo = sdf.format(new Date());

        String numString = String.format("%04d", pool.getNum());
        batchNo += numString;
        pool.setBatchNo(batchNo);
		/*
		RLock rLock = null;
		try{
			//加锁同步登录人数
			rLock = redissonClient.getLock("redisLock_" + key);
			rLock.lock(3, TimeUnit.SECONDS);
			//获取当前登录人数缓存中获取
			Object value = redisCache.get(key);
			Integer num = Integer.valueOf(String.valueOf(value));
			if(num == null){
				//获取不到查询数据库
				num = myExperienceMapper.getMaxLoginCount(poolId);
				if(num == null){
					num = 1;
				}
			}
			pool.setNum(num);
			num+=1;
			redisCache.set(key, num.toString(), 36000);
		} catch(Exception e){
			e.printStackTrace();
		} finally {
			//释放锁
			if(rLock != null){
				rLock.unlock();
			}
		}*/

        return pool;
    }

    /**
     * 内部封装 获取组别或者批次
     *
     * @param poolId
     * @param accountId
     * @param type
     * @return
     */
    private String getGroupOrBatchNo(Long poolId, Long accountId, Integer type) {
        ExperiencePoolAuthorize experiencePoolAuthorize = new ExperiencePoolAuthorize();
        experiencePoolAuthorize.setExperienceId(poolId);
        experiencePoolAuthorize.setRelationId(accountId);
        String s = null;
        List<ExperiencePoolAuthorize> authorizeList = experiencePoolAuthorizeService.selectList(new EntityWrapper<>(experiencePoolAuthorize));
        if (CollectionUtils.isNotEmpty(authorizeList)) {
            //获取组别
            if (type == 2) {
                if (null != authorizeList.get(0) && authorizeList.get(0).getGroup() != null) {
                    s = authorizeList.get(0).getGroup();
                }
                //获取批次
            } else if (type == 3) {
                if (null != authorizeList.get(0) && authorizeList.get(0).getBatchNo() != null) {
                    s = authorizeList.get(0).getBatchNo();
                }
            }

        }
        return s;
    }
}

