package com.yizhi.application.controller;


import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.domain.Product;
import com.yizhi.application.domain.StudentExperience;
import com.yizhi.application.service.IProductService;
import com.yizhi.application.service.IStudentExperienceService;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.custom.api.feign.PurchaseProductClient;
import com.yizhi.custom.api.param.PurchaseProductsParam;
import com.yizhi.custom.api.vo.custom.ProductVO;
import com.yizhi.custom.api.vo.domain.ProductVo;
import com.yizhi.core.application.cache.distributedlock.impl.RedisDistributedLock;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.point.application.feign.PointDetailsFeignClients;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-09-19
 */
@RestController
public class ProductController implements PurchaseProductClient {

    private static final Logger LOGGER = LoggerFactory.getLogger(ProductController.class);

    @Autowired
    private PointDetailsFeignClients pointDetailsFeignClients;

    @Autowired
    private IStudentExperienceService studentExperienceService;

    @Autowired
    private IProductService productService;

    @Autowired
    private RedisDistributedLock redisDistributedLock;

    @Override
    public ProductVO getProducts(Long poolId) {
        RequestContext res = ContextHolder.get();

        Integer totalPoint = pointDetailsFeignClients.queryPiont(res.getAccountId(),
                res.getCompanyId(), res.getSiteId(), res.getOrgId());
        if (null == totalPoint) {
            totalPoint = 0;
        }
        Long totalExperience = studentExperienceService.getTotalExperience(poolId, res.getAccountId());
        Product product = new Product();
        product.setCompanyId(res.getCompanyId());
        product.setSiteId(res.getSiteId());
        product.setState(1);
        List<Product> products = productService.selectList(new EntityWrapper<>(product));
        List<ProductVo> productVos=new ArrayList<>();
        for (Product p:products) {
            ProductVo pv=new ProductVo();
            BeanUtils.copyProperties(p,pv);
            productVos.add(pv);
        }

        ProductVO productVO = new ProductVO();
        productVO.setTotalPoint(totalPoint);
        productVO.setTotalExperience(totalExperience);
        if (CollectionUtils.isNotEmpty(products)) {
            productVO.setProducts(productVos);
        }
        return productVO;
    }

    @Override
    @RequestMapping(produces = "text/plain;charset=UTF-8")
    public String purchaseProducts(@RequestBody PurchaseProductsParam param) {

        List<ProductVo> products = param.getProducts();

        RequestContext res = ContextHolder.get();

        String purchaseProductName = "";
        String emptyProductName = "";
        if (CollectionUtils.isNotEmpty(products)) {
            for (ProductVo product : products) {
                String lockName = "product_" + product.getId();
                //获取redis锁
                if (redisDistributedLock.lock(lockName)) {
                    try {
                        Product product1 = productService.selectById(product.getId());
                        //如果库存足够,购买商品
                        if (product1.getRemainingAmount() >= product.getAmount()) {
                            StudentExperience studentExperience = new StudentExperience();
                            studentExperience.setActivityName(product1.getProductName());
                            studentExperience.setCompanyId(res.getCompanyId());
                            studentExperience.setSiteId(res.getSiteId());
                            //消费经验
                            studentExperience.setType(10);
                            studentExperience.setCreateById(res.getAccountId());
                            studentExperience.setCreateByName(res.getAccountName());
                            studentExperience.setState(1);
                            studentExperience.setAccountId(res.getAccountId());
                            studentExperience.setAccountName(res.getAccountName());
                            studentExperience.setExperiencePoolId(param.getPoolId());
                            studentExperience.setExperience(product1.getPrice() * product.getAmount());
                            studentExperienceService.insertStudentExperience(studentExperience);

                            //库存更新
                            product1.setRemainingAmount(product1.getRemainingAmount() - product.getAmount());
                            productService.updateById(product1);
                            if (StringUtils.isBlank(purchaseProductName)) {
                                purchaseProductName += product1.getProductName();
                            } else {
                                purchaseProductName += "," + product1.getProductName();
                            }
                        } else {
                            //库存不足,不能购买
                            if (StringUtils.isBlank(emptyProductName)) {
                                emptyProductName += product1.getProductName();
                            } else {
                                emptyProductName += "," + product1.getProductName();
                            }

                        }
                    } catch (Exception e) {
                        LOGGER.error("商品" + product.getId() + "购买异常", e);
                    } finally {
                        redisDistributedLock.releaseLock(lockName);
                        LOGGER.info("释放reids锁" + lockName);
                    }
                }
            }

            String result = null;
            if (StringUtils.isBlank(purchaseProductName)) {
                result = emptyProductName + "库存不足,未能购买";
            } else if (StringUtils.isBlank(emptyProductName)) {
                result = "购买成功";
            } else {
                result = purchaseProductName + "购买成功," + emptyProductName + "库存不足,未能购买";
            }
            System.out.println(result);
            return result;
        }
        return null;
    }
}

