package com.yizhi.application.controller;


import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.domain.ProjectActivityExperience;
import com.yizhi.application.domain.ProjectActivityExperienceSet;
import com.yizhi.application.service.IProjectActivityExperienceService;
import com.yizhi.application.service.IProjectActivityExperienceSetService;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.custom.api.vo.domain.ProjectActivityExperienceSetVo;
import com.yizhi.core.application.context.ContextHolder;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.yizhi.custom.api.feign.ProjectActivityExperienceSetClient;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author ding
 * @since 2018-11-06
 */
@RestController
@RequestMapping("/manage/experiencePool")
public class ProjectActivityExperienceSetController implements ProjectActivityExperienceSetClient {

    @Autowired
    private IProjectActivityExperienceSetService activityExperienceSetService;
    @Autowired
    private IProjectActivityExperienceService activityExperienceService;


    @Override
    @GetMapping("/projectActivityExperienceSet/list")
    public Map<String, Object> getProjectActivityExperienceSets(@RequestParam("activityExperienceId") Long activityExperienceId,
                                                                @RequestParam(value = "setType", required = false) Integer setType) {
        return activityExperienceSetService.getSets(activityExperienceId, setType);
    }

    @Override
    @GetMapping("/projectActivityExperienceSet/view")
    public ProjectActivityExperienceSetVo viewProjectActivityExperienceSet(Long activityExperienceSetId) {
        ProjectActivityExperienceSet pa=activityExperienceSetService.selectById(activityExperienceSetId);
        ProjectActivityExperienceSetVo pav=new ProjectActivityExperienceSetVo();
        BeanUtils.copyProperties(pa,pav);
        return pav;
    }

    @Override
    @GetMapping("/projectActivityExperienceSet/insert")
    public Map<String, Object> insertProjectActivityExperienceSet(@RequestParam(value = "activityExperienceSetId", required = false) Long activityExperienceSetId,
                                                                  @RequestParam(value = "activityExperienceId", required = true) Long activityExperienceId,
                                                                  @RequestParam(value = "setType", required = true) Integer setType,
                                                                  @RequestParam(value = "minScore", required = true) Integer minScore,
                                                                  @RequestParam(value = "experience", required = true) Long experience) {
        return activityExperienceSetService.insertSet(activityExperienceSetId, activityExperienceId, setType, minScore, experience);
    }

    @Override
    @GetMapping("/projectActivityExperienceSet/insertWithScore")
    public Map<String, Object> insertProjectActivityExperienceSetWithScore(@RequestParam(value = "activityExperienceSetId", required = false) Long activityExperienceSetId,
                                                                           @RequestParam(value = "activityExperienceId", required = true) Long activityExperienceId,
                                                                           @RequestParam(value = "setType", required = false) Integer setType,
                                                                           @RequestParam(value = "times", required = false) Integer times) {
        return activityExperienceSetService.insertSetWithScore(activityExperienceSetId, activityExperienceId, setType, times);
    }

    @Override
    @GetMapping("/projectActivityExperienceSet/delete")
    public boolean deleteProjectActivityExperienceSet(Long projectActivityExperienceSetId) {

        RequestContext res = ContextHolder.get();

        ProjectActivityExperienceSet projectActivityExperienceSet = new ProjectActivityExperienceSet();
        projectActivityExperienceSet = activityExperienceSetService.selectById(projectActivityExperienceSetId);

        boolean isOk = activityExperienceSetService.deleteById(projectActivityExperienceSetId);


        ProjectActivityExperienceSet projectActivityExperienceSet1 = new ProjectActivityExperienceSet();
        projectActivityExperienceSet1.setProjectActivityExperienceId(projectActivityExperienceSet.getProjectActivityExperienceId());
        EntityWrapper<ProjectActivityExperienceSet> entityWrapper = new EntityWrapper<>(projectActivityExperienceSet1);
        List<ProjectActivityExperienceSet> list = activityExperienceSetService.selectList(entityWrapper);
        if (CollectionUtils.isEmpty(list)) {
            ProjectActivityExperience projectActivityExperience = new ProjectActivityExperience();
            projectActivityExperience.setId(projectActivityExperienceSet.getProjectActivityExperienceId());
            projectActivityExperience.setActivityExperienceSource(0);
            projectActivityExperience.setUpdateById(res.getAccountId());
            projectActivityExperience.setUpdateByName(res.getAccountFullName());
            projectActivityExperience.setUpdateTime(new Date());
            activityExperienceService.updateById(projectActivityExperience);
        }

        return isOk;

    }

    @Override
    @GetMapping("/projectActivityExperienceSets/delete")
    public Map<String, Object> deleteProjectActivityExperienceSets(Long activityExperienceId) {

        RequestContext res = ContextHolder.get();

        Map<String, Object> map = new HashMap<>();
        ProjectActivityExperienceSet activityExperienceSet = new ProjectActivityExperienceSet();
        activityExperienceSet.setProjectActivityExperienceId(activityExperienceId);
        EntityWrapper<ProjectActivityExperienceSet> entityWrapper = new EntityWrapper<>(activityExperienceSet);
        boolean isOk = activityExperienceSetService.delete(entityWrapper);
        if (isOk) {
            ProjectActivityExperience projectActivityExperience = new ProjectActivityExperience();
            projectActivityExperience.setId(activityExperienceId);
            projectActivityExperience.setActivityExperienceSource(0);
            projectActivityExperience.setUpdateById(res.getAccountId());
            projectActivityExperience.setUpdateByName(res.getAccountFullName());
            projectActivityExperience.setUpdateTime(new Date());
            activityExperienceService.updateById(projectActivityExperience);
            map.put("activityExperienceId", activityExperienceId);
            map.put("source", 0);
            return map;
        }
        return null;
    }

}

