package com.yizhi.application.controller;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.sign.application.feign.SignApiClient;
import com.yizhi.sign.application.feign.SignRecordApiClient;
import com.yizhi.sign.application.vo.SignVO;
import com.yizhi.training.application.feign.TrainingProjectClient;
import com.yizhi.training.application.vo.domain.TpAuthorizationRangeVo;
import com.yizhi.training.application.vo.domain.TrainingProjectVo;
import com.yizhi.util.application.constant.QueueConstant;
import com.yizhi.util.application.constant.TpActivityType;
import com.yizhi.util.application.domain.Response;
import com.yizhi.util.application.event.TrainingProjectEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.yizhi.core.application.context.ContextHolder;


import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping("/api/sign")
public class StudentSignController {

    @Autowired
    private SignApiClient signClient;
    @Autowired
    SignRecordApiClient signRecordApiClient;
    @Autowired
    private TrainingProjectClient trainingProjectClient;
    @Autowired
    private TaskExecutor taskExecutor;
    @Autowired
    private CloudEventPublisher cloudEventPublisher;
    
    private static final Logger LOG = LoggerFactory.getLogger(StudentSignController.class);

    @ApiOperation(value = "签到（扫码进入）", notes = "签到（扫码进入）")
    @PostMapping(value = "/custom/scan")
    public Response<Object> customSignScan(@RequestBody SignVO signVO) {
        RequestContext requestContext = ContextHolder.get();
        signVO.setSignType(1);
        Long projectId = signVO.getTrainingProjectId();
        List<Long> relationIds = requestContext.getRelationIds();
        Long siteId = requestContext.getSiteId();
        Long accountId = requestContext.getAccountId();
        Map<String, String> signMap = new HashMap<String, String>();
        boolean isPass = false;
            TrainingProjectVo project = trainingProjectClient.getOne(projectId);
            Integer range = project.getVisibleRange();
            if (range != null && range == 1) {
                if (siteId != null && project.getSiteId() != null && siteId.equals(project.getSiteId())) {
                    isPass = true;
                }
            } else if (range != null && range == 0 && siteId.equals(project.getSiteId())) {
                List<TpAuthorizationRangeVo> tpAuthorizationRanges = trainingProjectClient.VisibleRange(projectId);
                List<Long> rangeIds = new ArrayList<>();
                if (tpAuthorizationRanges != null && tpAuthorizationRanges.size() > 0) {
                    rangeIds = tpAuthorizationRanges.stream().map(TpAuthorizationRangeVo::getRelationId).collect(Collectors.toList());
                    LOG.info("rangeIds" + rangeIds.toString());
                }
                if (rangeIds.contains(accountId)) {
                    isPass = true;
                } else {
                    for (Long relationId : relationIds) {
                        if (rangeIds.contains(relationId)) {
                            isPass = true;
                            break;
                        }
                    }
                }
            }
        if (isPass) {
            return sign(signVO);
        } else {
            return Response.fail("4001", "签到失败,您不在该项目范围内");
        }
    }
    @ApiOperation(value = "签到", notes = "签到")
    @PostMapping(value = "/")
    public Response<Object> sign(@RequestBody SignVO signVO) {
        RequestContext requestContext = ContextHolder.get();
        Long accountId = requestContext.getAccountId();
        Map<String, String> signMap = new HashMap<String, String>();
        boolean isPass = false;
        if (null == requestContext) {
            signMap.put("code", "0");
            signMap.put("name", "非法签到");
            return Response.ok(signMap);
        }
        signVO.setAccountId(accountId);
        // 查询是否签到
        String result = signClient.sign(signVO);
        if (result.indexOf("成功") >= 0) {
            signMap.put("code", "1");
            signMap.put("name", result);
        } else {
            signMap.put("code", "0");
            signMap.put("name", result);
        }
        LOG.info(result);

        // 向培训项目发送消息，告知业务已经完成
        try {
            EventWrapper<TrainingProjectEvent> eventWrapper = new EventWrapper<TrainingProjectEvent>(signVO.getId(), TrainingProjectEvent.getInstance(signVO.getId(), TpActivityType.TYPE_SIGN, accountId, new Date(), requestContext.getSiteId()));
            LOG.info("向培训项目发送消息，告知签到业务已经完成:{}", eventWrapper);
            taskExecutor.asynExecute(new AbstractTaskHandler() {
                @Override
                public void handle() {
                    try {
                        cloudEventPublisher.publish(QueueConstant.TRAINING_PROJECT_EVENT_QUEUE, eventWrapper);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            });
        } catch (Exception e) {
            LOG.info("签到业务向培训项目发送消息告知已经完成，失败：{}", e);
        }

        return Response.ok(signMap);
    }

}
