package com.yizhi.application.listener;

import com.baomidou.mybatisplus.mapper.EntityWrapper;

import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.course.application.feign.RecordeClient;
import com.yizhi.application.constant.PoolConstant;
import com.yizhi.application.domain.ExperienceRecord;
import com.yizhi.application.domain.StudentExperience;
import com.yizhi.application.redis.RedisUtils;
import com.yizhi.application.service.IExperienceRecordService;
import com.yizhi.application.service.IStudentExperienceService;
import com.yizhi.custom.api.vo.custom.ExamRuleListVo;
import com.yizhi.custom.api.vo.custom.ExperiencePoolExamRuleVo;
import com.yizhi.custom.api.vo.custom.ExperiencePoolRuleVo;

import com.yizhi.point.application.vo.PointParamVO;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.vo.AccountVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;

@Component
@Transactional
public class ExperienceListener {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExperienceListener.class);

    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private IStudentExperienceService studentExperienceService;
    @Autowired
    private RecordeClient recordeClient;
    @Autowired
    private IExperienceRecordService experienceRecordService;
    @Autowired
    private AccountClient accountClient;

	/*static {
		RedissonManager.init(null, "1");
	}*/

    @RabbitListener(queues = "experienceCourse")
    public void processCourse(String pointId) {
        LOGGER.info("------------处理消息course-----------------");
        handExperience(pointId, PoolConstant.COURSETYPE);
    }

    @RabbitListener(queues = "experienceResearch")
    public void processResearch(String pointId) {
        LOGGER.info("------------处理消息research-----------------");
        handExperience(pointId, PoolConstant.RESEARCHTYPE);
    }

    @RabbitListener(queues = "experienceExam")
    public void processExam(String pointId) {
        LOGGER.info("------------处理消息exam-----------------");
        handExperience(pointId, PoolConstant.EXAMTYPE);
    }

    @RabbitListener(queues = "experienceVote")
    public void processVote(String pointId) {
        LOGGER.info("------------处理消息vote-----------------");
        handExperience(pointId, PoolConstant.VOTETYPE);
    }

    @RabbitListener(queues = "experienceAssignment")
    public void processAssignment(String pointId) {
        LOGGER.info("------------处理消息vote-----------------");
        handExperience(pointId, PoolConstant.ASSIGNMENTTYPE);
    }

    //处理经验(按设定值)
    public boolean handExperience(String pointId, Integer type) {
        PointParamVO vo = null;
        try {
            if (null == pointId) {
                LOGGER.info("积分MQId为空");
                return false;
            }

            vo = (PointParamVO) redisUtils.get(pointId);
            if (vo == null) {
                LOGGER.info("消息ID{}获取不到缓存信息");
                return false;
            }
            LOGGER.info("开始处理消息ID{},业务ID{} 的经验", pointId, vo.getSourceId());
            Long activityId = null;
            //如果是课程发放经验,先取courseId(按章节发送的消息),没有再取sourseId(按课程发送的消息)
            if (type.equals(PoolConstant.COURSETYPE)) {
                if (null != vo.getCourseId()) {
                    activityId = vo.getCourseId();
                } else {
                    activityId = vo.getSourceId();
                }
            } else {
                activityId = vo.getSourceId();
            }
            Long accountId = vo.getAccountId();
            Long compantyId = vo.getCompanyId();
            Long siteId = vo.getSiteId();
            if (type.equals(PoolConstant.COURSETYPE)) {
                LOGGER.info("*************portRule" + vo.getPortRule());
                if (vo.getPortRule() == 2 || vo.getPortRule() == 3) {
                    try {
                        //睡眠5s
                        Thread.sleep(5000);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                    Long isFinished = recordeClient.getCourseFinished(accountId, vo.getSourceId());
                    LOGGER.info("isFinished" + isFinished);
                    if (isFinished == 0) {
                        LOGGER.info("课程{}未完成,不能发放经验", vo.getSourceId());
                        return false;
                    } else {
                        activityId = isFinished;
                    }
                }
                AccountVO accountVO = accountClient.findById(accountId);
                if (null != accountVO) {
                    vo.setOrgId(accountVO.getOrgId());
                }
            }

            //如果是考试
            if (type.equals(PoolConstant.EXAMTYPE) || type.equals(PoolConstant.ASSIGNMENTTYPE)) {
                List<ExperiencePoolExamRuleVo> examRuleVoList = studentExperienceService.getExamRules(activityId, compantyId, siteId, accountId, type);
                if (!CollectionUtils.isEmpty(examRuleVoList)) {
                    for (ExperiencePoolExamRuleVo examRuleVo : examRuleVoList) {
                        //按设定值发放跟其他活动一样
                        if (examRuleVo.getExperienceType() == 1) {
                            ExperiencePoolRuleVo experiencePoolRuleVo = new ExperiencePoolRuleVo();
                            experiencePoolRuleVo.setExperiencePoolId(examRuleVo.getExperiencePoolId());
                            experiencePoolRuleVo.setType(examRuleVo.getType());
                            experiencePoolRuleVo.setBizId(examRuleVo.getBizId());
                            experiencePoolRuleVo.setBizName(examRuleVo.getBizName());
                            experiencePoolRuleVo.setExperience(examRuleVo.getRuleList().get(0).getExperience());
                            addExperience(experiencePoolRuleVo, vo, type);
                        } else {
                            //按考试成绩发放
                            addScoreExperience(examRuleVo, vo, type);
                        }
                    }
                } else {
                    LOGGER.error("该站点未找到{}的经验发放规则", activityId);
                }
            } else {
                List<ExperiencePoolRuleVo> ruleVoList = studentExperienceService.getRules(activityId, compantyId, siteId, accountId);
                if (!CollectionUtils.isEmpty(ruleVoList)) {
                    for (ExperiencePoolRuleVo ruleVo : ruleVoList) {
                        addExperience(ruleVo, vo, type);
                    }
                    return true;
                } else {
                    LOGGER.error("该站点未找到{}的经验发放规则", activityId);
                }
            }
        } catch (Exception e) {
            LOGGER.error("发放经验异常, 活动ID{}", e);
            e.printStackTrace();
        }
        return false;
    }

    //按设定值发放经验
    public boolean addExperience(ExperiencePoolRuleVo ruleVo, PointParamVO vo, Integer type) {
        Long bizId = ruleVo.getBizId();
        Long poolId = ruleVo.getExperiencePoolId();
        String bizName = ruleVo.getBizName();
        ExperienceRecord experienceRecord = new ExperienceRecord();
        experienceRecord.setExperiencePoolId(poolId);
        experienceRecord.setActivityType(type);
        experienceRecord.setActivityId(bizId);
        experienceRecord.setAccountId(vo.getAccountId());
        EntityWrapper<ExperienceRecord> entityWrapper = new EntityWrapper<>(experienceRecord);
        ExperienceRecord experienceRecord1 = experienceRecordService.selectOne(entityWrapper);
        if (null != experienceRecord1) {
            //已经发放经验的查看记录,判断是否补发经验
            if (experienceRecord1.getExperience() >= ruleVo.getExperience()) {
                LOGGER.info("经验池{}已发放经验,无需补发", ruleVo.getExperiencePoolId());
            } else {
                Long oldExperience = Long.valueOf(experienceRecord1.getExperience());
                experienceRecord1.setExperience(Math.toIntExact(ruleVo.getExperience()));
                experienceRecord1.setUpdateTime(new Date());
                experienceRecord1.setUpdateById(vo.getAccountId());
                experienceRecord1.setUpdateByName(vo.getAccountName() == null ? "" : vo.getAccountName());
                experienceRecordService.updateById(experienceRecord1);
                //学生经验表插入补发经验数据
                StudentExperience studentExperience = new StudentExperience();
                studentExperience.setExperiencePoolId(poolId);
                studentExperience.setAccountId(vo.getAccountId());
                studentExperience.setAccountName(vo.getAccountName() == null ? "" : vo.getAccountName());
                studentExperience.setType(type);
                studentExperience.setActivityId(bizId);
                studentExperience.setActivityName(bizName);
                studentExperience.setCompanyId(vo.getCompanyId());
                studentExperience.setSiteId(vo.getSiteId());
                studentExperience.setOrgId(vo.getOrgId());
                studentExperience.setExperience(ruleVo.getExperience() - oldExperience);
                studentExperience.setCreateById(vo.getAccountId());
                studentExperience.setCreateTime(new Date());
                studentExperience.setCreateByName(vo.getAccountName() == null ? "" : vo.getAccountName());
                studentExperienceService.insertStudentExperience(studentExperience);
                LOGGER.info("活动{}已发放经验,补发经验", vo.getActivityName());
            }
        } else {
            //未发放经验的直接发放
            experienceRecord.setCreateTime(new Date());
            experienceRecord.setCreateById(vo.getAccountId());
            experienceRecord.setCreateByName(vo.getAccountName() == null ? "" : vo.getAccountName());
            experienceRecord.setActivityName(ruleVo.getBizName());
            experienceRecord.setExperience(Math.toIntExact(ruleVo.getExperience()));
            experienceRecordService.insertRecord(experienceRecord);
            //学员经验表插入发放数据
            StudentExperience studentExperience = new StudentExperience();
            studentExperience.setExperiencePoolId(poolId);
            studentExperience.setAccountId(vo.getAccountId());
            studentExperience.setAccountName(vo.getAccountName());
            studentExperience.setCompanyId(vo.getCompanyId());
            studentExperience.setSiteId(vo.getSiteId());
            studentExperience.setOrgId(vo.getOrgId());
            studentExperience.setType(type);
            studentExperience.setActivityId(bizId);
            studentExperience.setActivityName(bizName);
            studentExperience.setExperience(ruleVo.getExperience());
            studentExperience.setCreateById(vo.getAccountId());
            studentExperience.setCreateTime(new Date());
            studentExperience.setCreateByName(vo.getAccountName() == null ? "" : vo.getAccountName());
            studentExperienceService.insertStudentExperience(studentExperience);
            LOGGER.info("活动{}发放经验完成", vo.getActivityName());
        }
        return true;
    }

    //按考试分数发放经验
    public boolean addScoreExperience(ExperiencePoolExamRuleVo examRuleVo, PointParamVO vo, Integer type) {
        Long bizId = examRuleVo.getBizId();
        Long poolId = examRuleVo.getExperiencePoolId();
        String bizName = examRuleVo.getBizName();
        Integer score = vo.getScore();
        LOGGER.info("**************************score", score);
        LOGGER.info("**************************score:{}", score);
        Long experience = null;
        List<ExamRuleListVo> ruleList = examRuleVo.getRuleList();
        if (examRuleVo.getExperienceType() == 2) {
            if (!CollectionUtils.isEmpty(ruleList)) {
                for (ExamRuleListVo examRuleListVo : ruleList) {
                    if (score >= examRuleListVo.getMinScore() && null != examRuleListVo.getMaxScore() && score < examRuleListVo.getMaxScore()) {
                        experience = examRuleListVo.getExperience();
                    } else if (score >= examRuleListVo.getMinScore() && null != examRuleListVo.getMaxScore() && score > examRuleListVo.getMaxScore()) {
                        continue;
                    } else if (score >= examRuleListVo.getMinScore() && null == examRuleListVo.getMaxScore()) {
                        experience = examRuleListVo.getExperience();
                    }
                }
            }
        } else {
            if (!CollectionUtils.isEmpty(ruleList)) {
                Integer times = ruleList.get(0).getTimes();
                experience = Long.valueOf(score * times);
            }
        }
        ExperienceRecord experienceRecord = new ExperienceRecord();
        experienceRecord.setExperiencePoolId(poolId);
        experienceRecord.setActivityType(type);
        experienceRecord.setActivityId(bizId);
        experienceRecord.setAccountId(vo.getAccountId());
        EntityWrapper<ExperienceRecord> entityWrapper = new EntityWrapper<>(experienceRecord);
        ExperienceRecord experienceRecord1 = experienceRecordService.selectOne(entityWrapper);
        if (null == experience) {
            return false;
        }
        if (null != experienceRecord1) {
            //已经发放经验的查看记录,判断是否补发经验
            if (experienceRecord1.getExperience() >= experience) {
                LOGGER.info("经验池{}已发放经验,无需补发", poolId);
            } else {
                Integer oldExperience = experienceRecord1.getExperience();
                experienceRecord1.setExperience(Math.toIntExact(experience));
                experienceRecord1.setUpdateTime(new Date());
                experienceRecord1.setUpdateById(vo.getAccountId() == null ? 0 : vo.getAccountId());
                experienceRecord1.setUpdateByName(vo.getAccountName() == null ? "" : vo.getAccountName());
                experienceRecordService.updateById(experienceRecord1);
                //学生经验表插入补发经验数据
                StudentExperience studentExperience = new StudentExperience();
                studentExperience.setExperiencePoolId(poolId);
                studentExperience.setAccountId(vo.getAccountId());
                studentExperience.setAccountName(vo.getAccountName() == null ? "" : vo.getAccountName());
                studentExperience.setType(type);
                studentExperience.setActivityId(bizId);
                studentExperience.setActivityName(bizName);
                studentExperience.setExperience(experience - Long.valueOf(oldExperience));
                studentExperience.setCreateById(vo.getAccountId() == null ? 0 : vo.getAccountId());
                studentExperience.setCreateTime(new Date());
                studentExperience.setCreateByName(vo.getAccountName() == null ? "" : vo.getAccountName());
                studentExperienceService.insertStudentExperience(studentExperience);
                LOGGER.info("考试{}已发放经验,补发经验", vo.getActivityName());
            }
        } else {
            //未发放经验的直接发放
            experienceRecord.setCreateTime(new Date());
            experienceRecord.setCreateById(vo.getAccountId() == null ? 0 : vo.getAccountId());
            experienceRecord.setCreateByName(vo.getAccountName() == null ? "" : vo.getAccountName());
            experienceRecord.setActivityType(type);
            experienceRecord.setActivityName(vo.getActivityName() == null ? "" : vo.getActivityName());
            experienceRecord.setExperience(Math.toIntExact(experience));
            experienceRecordService.insertRecord(experienceRecord);
            //学员经验表插入发放数据
            StudentExperience studentExperience = new StudentExperience();
            studentExperience.setExperiencePoolId(poolId);
            studentExperience.setAccountId(vo.getAccountId());
            studentExperience.setAccountName(vo.getAccountName() == null ? "" : vo.getAccountName());
            studentExperience.setCompanyId(vo.getCompanyId());
            studentExperience.setSiteId(vo.getSiteId());
            studentExperience.setOrgId(vo.getOrgId());
            studentExperience.setType(type);
            studentExperience.setActivityId(bizId);
            studentExperience.setActivityName(bizName);
            studentExperience.setExperience(experience);
            studentExperience.setCreateById(vo.getAccountId());
            studentExperience.setCreateTime(new Date());
            studentExperience.setCreateByName(vo.getAccountName() == null ? "" : vo.getAccountName());
            studentExperienceService.insertStudentExperience(studentExperience);
            LOGGER.info("考试{}发放经验完成", vo.getActivityName());
        }
        return true;
    }

}
