package com.yizhi.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.domain.ExperiencePoolLevel;
import com.yizhi.application.domain.StudentExperiencePool;
import com.yizhi.custom.api.vo.custom.MyExperienceEventsVo;
import com.yizhi.custom.api.vo.custom.MyExperienceLevelVo;
import com.yizhi.custom.api.vo.custom.MyExperiencePoolVo;
import com.yizhi.custom.api.vo.custom.MyExperienceRankingVo;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 我的经验
 * </p>
 *
 * @author ding
 * @since 2018-11-06
 */
public interface MyExperienceMapper extends BaseMapper<StudentExperiencePool> {


    /**
     * 获取个人经验和等级
     *
     * @param poolId
     * @param accountId
     * @param sort
     * @return
     */
    MyExperienceLevelVo getMyExperiencePoolAndLevel(@Param("poolId") Long poolId, @Param("accountId") Long accountId);

    /**
     * 获取个人排名
     *
     * @param poolId
     * @param experience
     * @param batchNo
     * @param group
     * @return
     */
    Integer getMyExperienceRanking(@Param("poolId") Long poolId, @Param("experience") Integer experience, @Param("group") String group,@Param("batchNo") String batchNo);

    /**
     * 个人经验统计
     *
     * @param poolId
     * @param accountId
     * @param sort
     * @return
     */
    MyExperienceLevelVo getMyExperiencePool(@Param("poolId") Long poolId, @Param("accountId") Long accountId);

    /**
     * 获取单个经验等级
     *
     * @param poolId
     * @param experience
     * @return
     */
    ExperiencePoolLevel getMyExperienceLevel(@Param("poolId") Long poolId, @Param("experience") Integer experience);

    /**
     * 经验排行榜
     *
     * @param poolId
     * @param group
     * @param batchNo
     * @param pageNo
     * @param pageSize
     * @return
     */
    List<MyExperienceRankingVo> getMyExperienceRankings(@Param("poolId") Long poolId, @Param("group") String group, @Param("batchNo") String batchNo,
                                                        @Param("pageSize") Integer pageSize, @Param("pageNo") Integer pageNo);

    /**
     * 经验排行榜页数统计
     *
     * @param poolId
     * @param group
     * @param batchNo
     * @return
     */
    Integer getMyExperienceRankingsPageTotal(@Param("poolId") Long poolId, @Param("group") String group, @Param("batchNo") String batchNo);

    /**
     * 个人经验明细
     *
     * @param poolId
     * @param accountId
     * @param rowBounds
     * @return
     */
    List<MyExperienceEventsVo> getMyExperienceDtl(@Param("poolId") Long poolId, @Param("accountId") Long accountId,
                                                  @Param("page") Page<MyExperienceEventsVo> page);

    /**
     * 经验等级
     *
     * @param poolId
     * @return
     */
    List<MyExperienceLevelVo> getMyExperienceLevels(@Param("poolId") Long poolId);

    /**
     * 我的经验池信息
     *
     * @param poolId
     * @param accountId
     * @return
     */
    StudentExperiencePool getMyExperiencePoolInfo(@Param("poolId") Long poolId, @Param("accountId") Long accountId);

    /**
     * 我的经验池登录信息
     *
     * @param poolId
     * @param accountId
     * @return
     */
    MyExperiencePoolVo getMyExperienceLogin(@Param("poolId") Long poolId, @Param("accountId") Long accountId);

    /**
     * 获取最大登录人数
     *
     * @param poolId
     * @return
     */
    Integer getMaxLoginCount(@Param("poolId") Long poolId);

    List<MyExperienceEventsVo> getMyExperience(@Param("poolId") Long poolId, @Param("accountId") Long accountId);

    Map<String, Object> getBatchGroup(@Param("accountId") Long accountId, @Param("poolId") Long poolId);
}
