package com.yizhi.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.domain.ProjectActivityExperience;
import com.yizhi.application.domain.StudentExperience;
import com.yizhi.custom.api.vo.custom.ExperienceOrgRankingVO;
import com.yizhi.custom.api.vo.custom.ExperiencePoolReportExportVo;
import com.yizhi.custom.api.vo.custom.ExperiencePoolRuleVo;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 项目关联活动经验值表 Mapper 接口
 * </p>
 *
 * @author dingxiaowei123
 * @since 2018-11-12
 */
public interface StudentExperienceMapper extends BaseMapper<StudentExperience> {

    List<ExperiencePoolReportExportVo> getReportViewList(@Param("poolId") Long poolId, @Param("ids") List<Long> ids, Page<ExperiencePoolReportExportVo> page);

    List<ExperiencePoolRuleVo> getRules(@Param("activityId") Long activityId, @Param("compantyId") Long compantyId, @Param("siteId") Long siteId);

    ProjectActivityExperience getExamRules(@Param("poolId") Long poolId, @Param("activityId") Long activityId);

    /**
     * 自定义项目按部门经验排行
     *
     * @param poolId
     * @param page
     * @return
     */
    List<ExperienceOrgRankingVO> getOrgRankings(@Param("poolId") Long poolId, Page<ExperienceOrgRankingVO> page);

    /**
     * 自定义项目按部门获取单个部门排名
     *
     * @param poolId
     * @param totalExperience
     * @return
     */
    Integer getOrgRanking(@Param("poolId") Long poolId, @Param("totalExperience") Double totalExperience);

    /**
     * 获取按组别排名
     *
     * @param poolId
     * @param page
     * @return
     */
    List<ExperienceOrgRankingVO> getGroupRankings(Long poolId, Page<ExperienceOrgRankingVO> page);

    /**
     * 获取按组别排名每个组别名次
     *
     * @param poolId
     * @param totalExperience
     * @return
     */
    Integer getGroupRanking(@Param("poolId") Long poolId, @Param("totalExperience") Double totalExperience);

    /**
     * 获得按组别排名,可导入外部经验
     *
     * @param poolId
     * @param page
     * @return
     */
    List<ExperienceOrgRankingVO> getGroupRankingsWithImport(@Param("poolId") Long poolId, Page<ExperienceOrgRankingVO> page);

    /**
     * 经验池报表查看个人经验明细
     *
     * @param poolId
     * @param accountId
     * @param state
     * @param page
     * @return
     */
    List<ExperiencePoolReportExportVo> viewAccountDetails(@Param("poolId") Long poolId,
                                                          @Param("accountId") Long accountId,
                                                          @Param("state") Integer state, Page<ExperiencePoolReportExportVo> page);

    /**
     * 获取当页明细开始的总分
     * @param poolId
     * @param accountId
     * @param startIndex
     * @param endIndex
     * @return
     */
    Long getTotalExperience(@Param("poolId") Long poolId,
                            @Param("accountId") Long accountId,
                            @Param("startIndex") Integer startIndex,
                            @Param("endIndex") Integer endIndex);

    /**
     * 所有人经验变更明细/阶段明细/阶段消费明细
     * @param poolId
     * @param ids
     * @param startTime
     * @param endTime
     * @param state
     * @return
     */
    List<ExperiencePoolReportExportVo> viewAllDetails(@Param("poolId") Long poolId,
                                                      @Param("ids") List<Long> ids,
                                                      @Param("startTime") String startTime,
                                                      @Param("endTime") String endTime,
                                                      @Param("state") Integer state);

    /**
     * 获取阶段消耗总值,增加总值
     * @param accountIds
     * @param poolId
     * @param startTime
     * @param endTime
     * @return
     */
    @MapKey("accountId")
    Map<Long, Map<String, BigDecimal>> getPurchaseAndIncreaseExperienceTotal(@Param("accountIds") List<Long> accountIds,
                                                                             @Param("poolId") Long poolId,
                                                                             @Param("startTime") String startTime,
                                                                             @Param("endTime") String endTime);

    /**
     * 获得某个用户某个经验池的总经验
     * @param poolId
     * @param accountId
     * @return
     */
    Long getAccountTotalExperience(@Param("poolId") Long poolId,@Param("accountId") Long accountId);
}
