package com.yizhi.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.domain.*;
import com.yizhi.application.mapper.*;
import com.yizhi.application.orm.hierarchicalauthorization.HQueryUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.service.IExperiencePoolProjectService;
import com.yizhi.application.service.IExperiencePoolService;
import com.yizhi.application.service.IProjectActivityExperienceService;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.custom.api.param.ExperiencePoolParam;

import com.yizhi.custom.api.vo.custom.*;
import com.yizhi.custom.api.vo.domain.ExperiencePoolAuthorizeVo;
import com.yizhi.custom.api.vo.domain.ExperiencePoolProjectVo;
import com.yizhi.custom.api.vo.domain.ProjectActivityExperienceVo;

import com.yizhi.core.application.context.ContextHolder;

import com.yizhi.system.application.constant.AuthzConstant;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.system.remote.OrganizationClient;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.vo.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author ding
 * @since 2018-11-06
 */
@Service
public class ExperiencePoolServiceImpl extends ServiceImpl<ExperiencePoolMapper, ExperiencePool> implements IExperiencePoolService {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExperiencePoolServiceImpl.class);

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ExperiencePoolMapper poolMapper;

    @Autowired
    private ExperiencePoolAuthorizeMapper authorizeMapper;

    @Autowired
    private ExperiencePoolProjectMapper poolProjectMapper;

    @Autowired
    private IExperiencePoolProjectService poolProjectService;

    @Autowired
    private IProjectActivityExperienceService activityExperienceService;

    @Autowired
    private AccountClient accountClient;

    @Autowired
    private MyExperienceMapper myExperienceMapper;

    @Autowired
    private StudentExperienceMapper studentExperienceMapper;

    @Autowired
    private ReportClient reportClient;

    @Autowired
    private ExperiencePoolAuthorizeMapper poolAuthorizeMapper;

    @Autowired
    private OrganizationClient organizationClient;

    @Override
    public Page<ExperiencePool> getExperiencePoolList(ExperiencePoolParam param) {
        Integer pageNo = param.getPageNo();
        Integer pageSize = param.getPageSize();
        Page<ExperiencePool> page = new Page<>(pageNo, pageSize);
        HQueryUtil.startHQ(ExperiencePool.class);
        List<ExperiencePool> list = poolMapper.getExperiencePoolList(param, page);
        if (CollectionUtils.isNotEmpty(list)) {
            page.setRecords(list);
        }
        return page;
    }

    @Override
    public ExperiencePoolVoAutorizeAndProjectVO getExperiencePoolView(Long poolId) {
        ExperiencePoolVoAutorizeAndProjectVO poolAutorizeAndProjectVO = new ExperiencePoolVoAutorizeAndProjectVO();
        ExperiencePool experiencePool = null;
        List<ExperiencePoolAuthorize> authorizeList = null;
        List<ExperiencePoolProject> poolProjectList = null;
        List<ProjectActivityExperience> activityExperienceList = null;
        RequestContext res = ContextHolder.get();

        //查找经验池主体以及可见范围,关联项目及活动经验,装入拓展类
        experiencePool = poolMapper.selectById(poolId);
        poolAutorizeAndProjectVO.setId(poolId);
        poolAutorizeAndProjectVO.setExperiencePoolNo(experiencePool.getExperiencePoolNo());
        poolAutorizeAndProjectVO.setExperiencePoolName(experiencePool.getExperiencePoolName());
        if (experiencePool.getVisibleRange() != null) {
            poolAutorizeAndProjectVO.setVisibleRange(experiencePool.getVisibleRange());
        }

        //如果是指定范围查出学员范围列表并装入拓展类
        if (experiencePool.getVisibleRange().equals(2)) {
            ExperiencePoolAuthorize experiencePoolAuthorize = new ExperiencePoolAuthorize();
            experiencePoolAuthorize.setExperienceId(poolId);
            EntityWrapper<ExperiencePoolAuthorize> entityWrapper = new EntityWrapper<>(experiencePoolAuthorize);
            authorizeList = authorizeMapper.selectList(entityWrapper);
            List<ExperiencePoolAuthorizeVo> en = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(authorizeList)) {
                for (ExperiencePoolAuthorize e : authorizeList) {
                    ExperiencePoolAuthorizeVo ex = new ExperiencePoolAuthorizeVo();
                    BeanUtils.copyProperties(e, ex);
                    en.add(ex);
                }
                poolAutorizeAndProjectVO.setExperiencePoolAuthorizes(en);
            }
        }

        //查找关联项目列表并装入拓展类
        ExperiencePoolProject experiencePoolProject = new ExperiencePoolProject();
        experiencePoolProject.setExperienceId(poolId);
        EntityWrapper<ExperiencePoolProject> poolProjectEntityWrapper = new EntityWrapper<>(experiencePoolProject);
        poolProjectEntityWrapper.orderBy("type", true);
        poolProjectList = poolProjectMapper.selectList(poolProjectEntityWrapper);
        List<ExperiencePoolProjectVo> ep = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(poolProjectList)) {
            for (ExperiencePoolProject e : poolProjectList) {
                ExperiencePoolProjectVo ex = new ExperiencePoolProjectVo();
                BeanUtils.copyProperties(e, ex);
                ep.add(ex);
            }
            poolAutorizeAndProjectVO.setPoolProjects(ep);
        }

        //查找关联项目活动经验列表并装入拓展类


        return poolAutorizeAndProjectVO;
    }

    @Override
    public Map<String, Object> insertPoolAuthorizeAndProject(ExperiencePoolVoAutorizeAndProjectVO poolAutorizeAndProjectVO) {
        Map<String, Object> resultMap = new HashMap<>();
        ExperiencePool experiencePool = null;
        ExperiencePool experiencePool1 = new ExperiencePool();
        experiencePool1.setExperiencePoolNo(poolAutorizeAndProjectVO.getExperiencePoolNo());
        EntityWrapper<ExperiencePool> entityWrapper1 = new EntityWrapper<>(experiencePool1);
        if (null != poolAutorizeAndProjectVO.getId()) {
            if (!this.selectById(poolAutorizeAndProjectVO.getId()).getExperiencePoolNo().equals(poolAutorizeAndProjectVO.getExperiencePoolNo())
                    && CollectionUtils.isNotEmpty(this.selectList(entityWrapper1))) {
                resultMap.put("isOk", false);
                return resultMap;
            } else {
                resultMap.put("isOk", true);
            }
        } else {
            if (CollectionUtils.isNotEmpty(this.selectList(entityWrapper1))) {
                resultMap.put("isOk", false);
                return resultMap;
            } else {
                resultMap.put("isOk", true);
            }
        }

        RequestContext res = ContextHolder.get();
        List<ExperiencePoolAuthorize> authorizeList = null;
        List<ExperiencePoolProject> poolProjectList = null;
        List<ProjectActivityExperience> activityExperienceList = null;
        Long id = null;
        //判断为新建还是修改
        if (null == poolAutorizeAndProjectVO.getId()) {
            //新建入库
            id = idGenerator.generate();
            experiencePool = new ExperiencePool();
            experiencePool.setId(id);
            experiencePool.setCreateById(res.getAccountId());
            experiencePool.setCreateByName(res.getAccountFullName());
            experiencePool.setCreateTime(new Date());
            experiencePool.setState(1);
            experiencePool.setExperiencePoolType(1);
            experiencePool.setExperiencePoolNo(poolAutorizeAndProjectVO.getExperiencePoolNo());
            experiencePool.setExperiencePoolName(poolAutorizeAndProjectVO.getExperiencePoolName());
            experiencePool.setVisibleRange(poolAutorizeAndProjectVO.getVisibleRange());
            experiencePool.setCompanyId(res.getCompanyId());
            experiencePool.setSiteId(res.getSiteId());
            experiencePool.setOrgId(res.getOrgId());
            poolMapper.insert(experiencePool);
        } else {
            //修改入库
            experiencePool = this.selectById(poolAutorizeAndProjectVO.getId());
            experiencePool.setExperiencePoolNo(poolAutorizeAndProjectVO.getExperiencePoolNo());
            experiencePool.setExperiencePoolName(poolAutorizeAndProjectVO.getExperiencePoolName());
            experiencePool.setVisibleRange(poolAutorizeAndProjectVO.getVisibleRange());
            experiencePool.setUpdateById(res.getAccountId());
            experiencePool.setUpdateByName(res.getAccountFullName());
            experiencePool.setUpdateTime(new Date());
            this.updateById(experiencePool);
        }


        //入库关联项目及有关联项目活动经验
        if (null == id) {
            id = experiencePool.getId();
        }

        PoolSaveReturnVO poolSaveReturnVO = new PoolSaveReturnVO();
        if (CollectionUtils.isNotEmpty(poolAutorizeAndProjectVO.getPoolProjects())) {
            //有关联项目对比更新入库
            List<ExperiencePoolProjectVo> ep = poolAutorizeAndProjectVO.getPoolProjects();
            List<ExperiencePoolProject> epx = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(ep)) {
                for (ExperiencePoolProjectVo e : ep) {
                    ExperiencePoolProject ex = new ExperiencePoolProject();
                    BeanUtils.copyProperties(e, ex);
                    epx.add(ex);
                }
            }
            activityExperienceList = poolProjectService.saveList(id, epx);
        } else {
            //没有关联项目对比删除旧数据
            ExperiencePoolProject experiencePoolProject = new ExperiencePoolProject();
            experiencePoolProject.setExperienceId(id);
            EntityWrapper<ExperiencePoolProject> entityWrapper = new EntityWrapper<>(experiencePoolProject);
            List<ExperiencePoolProject> list = poolProjectService.selectList(entityWrapper);
            poolProjectService.delete(entityWrapper);
            List<Long> deleteList = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(list)) {
                deleteList = list.stream().map(ExperiencePoolProject::getId).collect(Collectors.toList());
                for (Long poolProjectId : deleteList) {
                    ProjectActivityExperience projectActivityExperience = new ProjectActivityExperience();
                    projectActivityExperience.setExperiencePoolProjectId(poolProjectId);
                    EntityWrapper<ProjectActivityExperience> deleteWrapper = new EntityWrapper<>(projectActivityExperience);
                    activityExperienceService.delete(deleteWrapper);
                }
            }
            poolSaveReturnVO.setPoolId(id);
            resultMap.put("poolSaveReturnVO", poolSaveReturnVO);
            return resultMap;
        }
        Map<Long, List<ProjectActivityExperience>> map = new HashMap<>();
        poolSaveReturnVO.setPoolId(id);
        List<ProjectActivityExperienceVo> exp = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(activityExperienceList)) {
            for (ProjectActivityExperience e : activityExperienceList) {
                ProjectActivityExperienceVo ex = new ProjectActivityExperienceVo();
                BeanUtils.copyProperties(e, ex);
                exp.add(ex);
            }
            poolSaveReturnVO.setActivityExperienceList(exp);
        }
        resultMap.put("poolSaveReturnVO", poolSaveReturnVO);
        return resultMap;
    }

    @Override
    public boolean complete(Long poolId) {
        RequestContext res = ContextHolder.get();
        ExperiencePool experiencePool = new ExperiencePool();
        experiencePool = this.selectById(poolId);
        experiencePool.setState(2);
        experiencePool.setUpdateById(res.getAccountId());
        experiencePool.setUpdateByName(res.getAccountFullName());
        experiencePool.setUpdateTime(new Date());
        return this.updateById(experiencePool);
    }

    @Override
    public Page<ExperiencePoolReportVo> getPoolReportList(String poolName, Integer pageNo, Integer pageSize, RequestContext requestContext) {

        Long companyId = requestContext.getCompanyId();
        Long siteId = requestContext.getSiteId();

        Page<ExperiencePoolReportVo> page = new Page<>(pageNo, pageSize);

        List<Long> accountIds = new ArrayList<>();
        if (!requestContext.isAdmin()) {
            HQAccountInManageParam hqAccountInManageParam = new HQAccountInManageParam();
            hqAccountInManageParam.setModuleType(AuthzConstant.moduleType.statistics_experience);
            hqAccountInManageParam.setSiteId(siteId);
            hqAccountInManageParam.setAccountId(requestContext.getAccountId());
            ReportAccountRespInManageVO respInManageVO = reportClient.getRangeAccountsInManage(hqAccountInManageParam);
            if (null != respInManageVO) {
                List<ReportAccountRespVO> accountList = respInManageVO.getList();
                if (CollectionUtils.isNotEmpty(accountList)) {
                    accountIds = accountList.stream().map(ReportAccountRespVO::getUserId).collect(Collectors.toList());
                }
            }
        }
        //优化,如果是管理员不要将用户id加入限制条件
//        else {
//            ReportRangeAccountReqVO reportRangeAccountReqVO = new ReportRangeAccountReqVO();
//            reportRangeAccountReqVO.setSiteId(siteId);
//            List<ReportAccountRespVOVo> accountList = reportClient.getRangeAccounts(reportRangeAccountReqVO);
//            if (CollectionUtils.isNotEmpty(accountList)) {
//                accountIds = accountList.stream().map(ReportAccountRespVOVo::getUserId).collect(Collectors.toList());
//            }
//        }
        String name = null;
        if (StringUtils.isNotBlank(poolName)) {
            name = poolName.trim();
        }

        Integer start = (pageNo - 1) * pageSize;

        List<ExperiencePoolReportVo> list = new ArrayList<>();

        Connection conn = null;
        Statement st = null;
        Statement st1 = null;

        ResultSet rs = null;
        ResultSet countRs = null;
        StringBuilder sql = new StringBuilder("");
        StringBuilder countSql = new StringBuilder();

        sql.append("SELECT e.id as experiencePoolId,e.experience_pool_name ,e.visible_range as visibleRange,")
                .append("e.create_time as poolCreateTime,IFNULL(b.num,0) as rangeAcountNum")
                .append(" FROM experience_pool e")
                .append(" LEFT JOIN (select experience_id , COUNT(relation_id) as num from experience_pool_authorize")
                .append(" where 1=1");
        if (CollectionUtils.isNotEmpty(accountIds)) {
            StringBuilder s = new StringBuilder();
            for (Long c : accountIds) {
                int i = accountIds.size() - 1;
                if (accountIds.indexOf(c) != i) {
                    s.append(c).append(",");
                } else {
                    s.append(c);
                }

            }
            System.out.println(s);
            sql.append(" and relation_id in(").append(s).append(")");
        }
        sql.append(" GROUP BY\n" +
                "        experience_id)b\n" +
                "        on e.id = b.experience_id and e.company_id =").append(companyId).append(" and e.site_id = ")
                .append(siteId);
        sql.append(" where e.company_id = ").append(companyId).append(" and e.site_id = ").append(siteId);
        if (StringUtils.isNotBlank(name)) {
            sql.append(" and e.experience_pool_name like CONCAT('%','").append(name).append("','%')");
        }
        sql.append(" and e.state = 2 ORDER BY e.create_time DESC,e.id limit ").append(start).append(",").append(pageSize);

        countSql.append("select count(id) as total from experience_pool where company_id =")
                .append(companyId).append(" and site_id =").append(siteId).append(" and state = 2");
        if (StringUtils.isNotBlank(name)) {
            countSql.append(" and experience_pool_name like CONCAT('%','").append(name).append("','%')");
        }
        Integer pageTotal = 0;


        try {
            try {
                conn = this.sqlSessionBatch().getConnection();
                st = conn.createStatement();
                LOGGER.info("执行的sql={}", sql.toString());
                rs = st.executeQuery(sql.toString());
                while (rs.next()) {
                    ExperiencePoolReportVo reportVo = new ExperiencePoolReportVo();
                    reportVo.setExperiencePoolId(rs.getLong("experiencePoolId"));
                    reportVo.setExperiencePoolName(rs.getString("experience_pool_name"));
                    reportVo.setVisibleRange(rs.getInt("visibleRange"));
                    reportVo.setPoolCreateTime(rs.getDate("poolCreateTime"));
                    reportVo.setRangeAcountNum(rs.getInt("rangeAcountNum"));
                    list.add(reportVo);
                }
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                if (null != rs) {
                    rs.close();
                }
                if (null != st) {
                    st.close();
                }
                if (null != conn) {
                    conn.close();
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
            LOGGER.info("经验池列表创建jdbc连接过程中,获取列表信息出现异常");
        }

        try {
            try {
                conn = this.sqlSessionBatch().getConnection();
                st1 = conn.createStatement();
                LOGGER.info("执行的countSql={}", countSql.toString());
                countRs = st1.executeQuery(countSql.toString());

                while (countRs.next()) {
                    pageTotal = countRs.getInt("total");
                }
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                if (null != countRs) {
                    countRs.close();
                }
                if (null != st1) {
                    st1.close();
                }
                if (null != conn) {
                    conn.close();
                }
            }
        } catch (SQLException e) {
            LOGGER.info("经验池列表创建jdbc连接过程中,获取分页total出现异常");
        }


        LOGGER.info("*****************找到{}个经验池" + list.size());
        //查出全平台可见人数
        ReportRangeAccountReqVO reqVO = new ReportRangeAccountReqVO();
        reqVO.setSiteId(siteId);
        Integer allNum = Math.toIntExact(reportClient.getRangeAccountCountNum(reqVO));
        Integer roleNum = accountIds.size();
        if (CollectionUtils.isNotEmpty(list)) {
            for (ExperiencePoolReportVo experiencePoolReportVo : list) {
                if (!requestContext.isAdmin() && CollectionUtils.isEmpty(accountIds)) {
                    experiencePoolReportVo.setRangeAcountNum(0);
                } else if (experiencePoolReportVo.getVisibleRange() == 1) {
                    if (requestContext.isAdmin()) {
                        experiencePoolReportVo.setRangeAcountNum(allNum);
                    } else {
                        experiencePoolReportVo.setRangeAcountNum(roleNum);
                    }
                }
            }
            page.setRecords(list);
            page.setTotal(pageTotal);
        }
        return page;
    }

    @Override
    public Page<ExperiencePoolReportExportVo> getReportViewList(Long poolId, String key, String orgName, Integer pageNo, Integer pageSize, Integer type, String startTime, String endTime, Long siteId, RequestContext requestContext) {

        if (StringUtils.isBlank(startTime)) {
            startTime = null;
        }
        if (StringUtils.isBlank(endTime)) {
            endTime = null;
        }

        Page<ExperiencePoolReportExportVo> page = new Page<>(pageNo, pageSize);
        //先分页查找对应关联用户id和总经验值

        StringBuilder sql = new StringBuilder();
        sql.append("select *\n" +
                "        from experience_pool e\n" +
                "        where id = ").append(poolId);

        Connection conn = this.sqlSessionBatch().getConnection();

        Integer visibleRange = 1;
        Statement st = null;
        ResultSet rs = null;
        try {
            st = conn.createStatement();
            LOGGER.info("执行的sql={}", sql.toString());
            rs = st.executeQuery(sql.toString());
            while (rs.next()) {

                visibleRange = rs.getInt("visible_range");

            }
            rs.close();
        } catch (Exception e) {
            e.printStackTrace();
            return page;
        } finally {
            if (null != rs) {
                try {
                    rs.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }
            if (null != st) {
                try {
                    st.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }
            if (null != conn) {
                try {
                    conn.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }
        }

        List<ExperiencePoolReportExportVo> list = null;
        if (visibleRange == 1) {

            boolean isGetAccount = true;
            List<ReportAccountRespVO> accountVOList = null;
            List<ReportAccountRespVOVo> accountRespVOVos = new ArrayList<>();
            GetAccountListVO getAccountListVO = getAccountList(requestContext, key, orgName, isGetAccount, accountVOList);
            if (null != getAccountListVO) {
                isGetAccount = getAccountListVO.getIsGetAccount();
                accountRespVOVos = getAccountListVO.getAccountVOList();
                if (!accountRespVOVos.isEmpty()) {
                    for (ReportAccountRespVOVo rvv : accountRespVOVos
                    ) {
                        ReportAccountRespVO rv = new ReportAccountRespVO();
                        BeanUtils.copyProperties(rvv, rv);
                        accountVOList.add(rv);
                    }
                }
            }

            List<Long> ids = new ArrayList<>();
            Map<Long, ReportAccountRespVO> accountVOMap = new HashMap<>();

            //如果全平台可见并且是管理员,先取出记录再根据记录取出对应用户信息,避免全平台人数过多,重复取出多余数据
            if (isGetAccount) {
                list = studentExperienceMapper.getReportViewList(poolId, null, page);
                if (CollectionUtils.isNotEmpty(list)) {
                    for (ExperiencePoolReportExportVo vo : list) {
                        ids.add(vo.getAccountId());
                    }
                    ReportRangeAccountReqVO reqVO = new ReportRangeAccountReqVO();
                    reqVO.setSiteId(siteId);
                    reqVO.setAccountIds(ids);
                    List<ReportAccountRespVO> respVOS = reportClient.getRangeAccounts(reqVO);
                    if (CollectionUtils.isNotEmpty(respVOS)) {
                        for (ReportAccountRespVO respVO : respVOS) {
                            accountVOMap.put(respVO.getUserId(), respVO);
                        }
                    }
                }
            } else {
                if (CollectionUtils.isNotEmpty(accountVOList)) {
                    for (ReportAccountRespVO reportAccountRespVO : accountVOList) {
                        ids.add(reportAccountRespVO.getUserId());
                        accountVOMap.put(reportAccountRespVO.getUserId(), reportAccountRespVO);
                    }
                    list = studentExperienceMapper.getReportViewList(poolId, ids, page);
                } else {
                    return page;
                }

            }
            if (CollectionUtils.isNotEmpty(list)) {
                Map<Long, Map<String, BigDecimal>> experienceMap = null;
                if (null != type && type == 1) {
                    List<Long> accountIds = list.stream().map(ExperiencePoolReportExportVo::getAccountId).collect(Collectors.toList());
                    if (CollectionUtils.isNotEmpty(accountIds)) {
                        experienceMap = studentExperienceMapper.getPurchaseAndIncreaseExperienceTotal(accountIds, poolId, startTime, endTime);
                    }
                }
                for (ExperiencePoolReportExportVo reportExportVo : list) {

                    ReportAccountRespVO account = accountVOMap.get(reportExportVo.getAccountId());
                    if (null == account) {
                        LOGGER.error("未找到该关联用户信息:" + reportExportVo.getAccountId());
                        continue;
                    }
                    reportExportVo.setAccountName(account.getUserName());
                    reportExportVo.setAccountFullName(account.getUserFullName());
                    reportExportVo.setOrgName(account.getOrgName());
                    reportExportVo.setOrgId(account.getOrgId());
                    reportExportVo.setEmail(account.getEmail());
                    reportExportVo.setAccountCreateTime(account.getCreateTime());
                    if (null != experienceMap) {
                        BigDecimal increase = null;
                        BigDecimal purchase = null;
                        if (null != experienceMap.get(reportExportVo.getAccountId())) {
                            increase = experienceMap.get(reportExportVo.getAccountId()).get("increaseTotal");
                            purchase = experienceMap.get(reportExportVo.getAccountId()).get("purchaseTotal");
                        }

                        Long increaseTotal = null;
                        Long purchaseTotal = null;
                        if (null != increase) {
                            increaseTotal = increase.longValue();
                        }
                        if (null != purchase) {
                            purchaseTotal = purchase.longValue();
                        }

                        reportExportVo.setPurchaseExperienceTotal(purchaseTotal == null ? 0 : purchaseTotal);
                        reportExportVo.setIncreaseExperienceTotal(increaseTotal == null ? 0 : increaseTotal);
                    }
                    if (type == null) {
                        if (null != reportExportVo.getExperienceTotal()) {
                            Integer experience = Math.toIntExact(reportExportVo.getExperienceTotal());
                            //查找经验等级装入实体类中
                            ExperiencePoolLevel experienceLevel = myExperienceMapper.getMyExperienceLevel(poolId, experience);
                            if (null != experienceLevel) {
                                reportExportVo.setExperienceLevel(experienceLevel.getLevelName());
                            }
//                        //将分类经验查找出来并装入实体类中
//                        List<MyExperienceEventsVo> experienceEventsVos = myExperienceMapper.getMyExperience(poolId, reportExportVo.getAccountId());
//                        if (CollectionUtils.isNotEmpty(experienceEventsVos)) {
//                            for (MyExperienceEventsVo myExperienceEventsVo : experienceEventsVos) {
//                                if (myExperienceEventsVo.getEventName().equals("课程")) {
//                                    Long courseExperience = myExperienceEventsVo.getExperience() == null ? 0 : Long.valueOf(myExperienceEventsVo.getExperience());
//                                    reportExportVo.setCourseExperience(courseExperience);
//                                } else if (myExperienceEventsVo.getEventName().equals("考试")) {
//                                    Long examExperience = myExperienceEventsVo.getExperience() == null ? 0 : Long.valueOf(myExperienceEventsVo.getExperience());
//                                    reportExportVo.setExamExperience(examExperience);
//                                } else if (myExperienceEventsVo.getEventName().equals("调研")) {
//                                    Long researchExperience = myExperienceEventsVo.getExperience() == null ? 0 : Long.valueOf(myExperienceEventsVo.getExperience());
//                                    reportExportVo.setResearchExperience(researchExperience);
//                                } else if (myExperienceEventsVo.getEventName().equals("投票")) {
//                                    Long voteExperience = myExperienceEventsVo.getExperience() == null ? 0 : Long.valueOf(myExperienceEventsVo.getExperience());
//                                    reportExportVo.setVoteExperience(voteExperience);
//                                } else if (myExperienceEventsVo.getEventName().equals("外部导入")) {
//                                    Long externalExperience = myExperienceEventsVo.getExperience() == null ? 0 : Long.valueOf(myExperienceEventsVo.getExperience());
//                                    reportExportVo.setExternalExperience(externalExperience);
//                                }
//                            }
//                        }
                        }
                    }
                }
            }


        } else if (visibleRange == 2) {

            //可见范围用户id
            List<Long> authorizeIds = null;
            //可见范围内的管辖区用户
            List<Long> ids = new ArrayList<>();
            ExperiencePoolAuthorize poolAuthorize = new ExperiencePoolAuthorize();
            poolAuthorize.setExperienceId(poolId);
            EntityWrapper<ExperiencePoolAuthorize> entityWrapper = new EntityWrapper<>(poolAuthorize);
            List<ExperiencePoolAuthorize> authorizeList = poolAuthorizeMapper.selectList(entityWrapper);

            if (CollectionUtils.isNotEmpty(authorizeList)) {
                authorizeIds = authorizeList.stream().map(ExperiencePoolAuthorize::getRelationId).collect(Collectors.toList());
                LOGGER.info("#########################authorizeIds.size:" + authorizeIds.size());
            }

            if (CollectionUtils.isEmpty(authorizeIds)) {
                return null;
            }

            List<ReportAccountRespVO> accountVOList = null;
            Map<Long, ReportAccountRespVO> accountsMap = new HashMap<>();
            if (requestContext.isAdmin()) {
                ReportRangeAccountReqVO reportRangeAccountReqVO = new ReportRangeAccountReqVO();
                reportRangeAccountReqVO.setSiteId(siteId);
                reportRangeAccountReqVO.setAccountIds(authorizeIds);
                if (StringUtils.isNotBlank(key)) {
                    reportRangeAccountReqVO.setAccountName(key.trim());
                }
                if (StringUtils.isNotBlank(orgName)) {
                    reportRangeAccountReqVO.setOrgName(orgName.trim());
                }
                accountVOList = reportClient.getRangeAccounts(reportRangeAccountReqVO);
                if (CollectionUtils.isNotEmpty(accountVOList)) {
                    for (ReportAccountRespVO reportAccountRespVO : accountVOList) {
                        ids.add(reportAccountRespVO.getUserId());
                        accountsMap.put(reportAccountRespVO.getUserId(), reportAccountRespVO);
                    }
                }
            } else {
                HQAccountInManageParam reportRangeAccountReqVO = new HQAccountInManageParam();
                reportRangeAccountReqVO.setAccountId(requestContext.getAccountId());
                reportRangeAccountReqVO.setSiteId(requestContext.getSiteId());
                reportRangeAccountReqVO.setModuleType(AuthzConstant.moduleType.statistics_experience);
                if (StringUtils.isNotBlank(key)) {
                    reportRangeAccountReqVO.setAccountName(key.trim());
                }
                if (StringUtils.isNotBlank(orgName)) {
                    reportRangeAccountReqVO.setOrgName(orgName.trim());
                }
                ReportAccountRespInManageVO reportAccountRespInManageVO = reportClient.getRangeAccountsInManage(reportRangeAccountReqVO);
                if (null != reportAccountRespInManageVO) {
                    if (!reportAccountRespInManageVO.getIsAdmin()) {
                        accountVOList = reportAccountRespInManageVO.getList();
                    } else {
                        ReportRangeAccountReqVO reportRangeAccountReqVO1 = new ReportRangeAccountReqVO();
                        reportRangeAccountReqVO1.setSiteId(siteId);
                        if (StringUtils.isNotBlank(key)) {
                            reportRangeAccountReqVO1.setAccountName(key.trim());
                        }
                        if (StringUtils.isNotBlank(orgName)) {
                            reportRangeAccountReqVO1.setOrgName(orgName.trim());
                        }
                        accountVOList = reportClient.getRangeAccounts(reportRangeAccountReqVO1);
                    }
                }

                if (CollectionUtils.isNotEmpty(accountVOList)) {
                    for (ReportAccountRespVO reportAccountRespVO : accountVOList) {
                        if (authorizeIds.contains(reportAccountRespVO.getUserId())) {
                            ids.add(reportAccountRespVO.getUserId());
                            accountsMap.put(reportAccountRespVO.getUserId(), reportAccountRespVO);
                        }
                    }
                }
            }


            if (CollectionUtils.isNotEmpty(ids)) {
                list = studentExperienceMapper.getReportViewList(poolId, ids, page);
                LOGGER.info("##########################经验列表list.size:" + list.size());
            } else {
                return page;
            }


            if (CollectionUtils.isNotEmpty(list)) {
                Map<Long, Map<String, BigDecimal>> experienceMap = null;
                if (null != type && type == 1) {
                    List<Long> accountIds = list.stream().map(ExperiencePoolReportExportVo::getAccountId).collect(Collectors.toList());
                    if (CollectionUtils.isNotEmpty(accountIds)) {
                        experienceMap = studentExperienceMapper.getPurchaseAndIncreaseExperienceTotal(accountIds, poolId, startTime, endTime);
                    }
                }
                //遍历ids将数据查找装入类中
                for (ExperiencePoolReportExportVo reportExportVo : list) {
                    //查找用记信息装入实体类中
                    ReportAccountRespVO accountVO = accountsMap.get(reportExportVo.getAccountId());
                    if (null == accountVO) {
                        LOGGER.error("未找到该关联用户信息:" + reportExportVo.getAccountId());
                        continue;
                    }
                    reportExportVo.setAccountFullName(accountVO.getUserFullName());
                    reportExportVo.setOrgName(accountVO.getOrgName());
                    reportExportVo.setOrgId(accountVO.getOrgId());
                    reportExportVo.setAccountName(accountVO.getUserName());
                    reportExportVo.setEmail(accountVO.getEmail());
                    reportExportVo.setAccountCreateTime(accountVO.getCreateTime());
                    if (null != experienceMap) {
                        BigDecimal increase = null;
                        BigDecimal purchase = null;
                        if (null != experienceMap.get(reportExportVo.getAccountId())) {
                            increase = experienceMap.get(reportExportVo.getAccountId()).get("increaseTotal");
                            purchase = experienceMap.get(reportExportVo.getAccountId()).get("purchaseTotal");
                        }
                        Long increaseTotal = null;
                        Long purchaseTotal = null;
                        if (null != increase) {
                            increaseTotal = increase.longValue();
                        }
                        if (null != purchase) {
                            purchaseTotal = purchase.longValue();
                        }

                        reportExportVo.setPurchaseExperienceTotal(purchaseTotal == null ? 0 : purchaseTotal);
                        reportExportVo.setIncreaseExperienceTotal(increaseTotal == null ? 0 : increaseTotal);
                    }

                    if (null == type) {
                        if (null != reportExportVo.getExperienceTotal()) {
                            Integer experience = Math.toIntExact(reportExportVo.getExperienceTotal());
                            //查找经验等级装入实体类中
                            ExperiencePoolLevel experienceLevel = myExperienceMapper.getMyExperienceLevel(poolId, experience);
                            if (null != experienceLevel) {
                                reportExportVo.setExperienceLevel(experienceLevel.getLevelName());
                            }
//                        //将分类经验查找出来并装入实体类中
//                        List<MyExperienceEventsVo> experienceEventsVos = myExperienceMapper.getMyExperience(poolId, reportExportVo.getAccountId());
//                        if (CollectionUtils.isNotEmpty(experienceEventsVos)) {
//                            for (MyExperienceEventsVo myExperienceEventsVo : experienceEventsVos) {
//                                if (myExperienceEventsVo.getEventName().equals("课程")) {
//                                    Long courseExperience = myExperienceEventsVo.getExperience() == null ? 0 : Long.valueOf(myExperienceEventsVo.getExperience());
//                                    reportExportVo.setCourseExperience(courseExperience);
//                                } else if (myExperienceEventsVo.getEventName().equals("考试")) {
//                                    Long examExperience = myExperienceEventsVo.getExperience() == null ? 0 : Long.valueOf(myExperienceEventsVo.getExperience());
//                                    reportExportVo.setExamExperience(examExperience);
//                                } else if (myExperienceEventsVo.getEventName().equals("调研")) {
//                                    Long researchExperience = myExperienceEventsVo.getExperience() == null ? 0 : Long.valueOf(myExperienceEventsVo.getExperience());
//                                    reportExportVo.setResearchExperience(researchExperience);
//                                } else if (myExperienceEventsVo.getEventName().equals("投票")) {
//                                    Long voteExperience = myExperienceEventsVo.getExperience() == null ? 0 : Long.valueOf(myExperienceEventsVo.getExperience());
//                                    reportExportVo.setVoteExperience(voteExperience);
//                                } else if (myExperienceEventsVo.getEventName().equals("外部导入")) {
//                                    Long externalExperience = myExperienceEventsVo.getExperience() == null ? 0 : Long.valueOf(myExperienceEventsVo.getExperience());
//                                    reportExportVo.setExternalExperience(externalExperience);
//                                }
//                            }
//                        }
                        }
                    }
                }
            }
        }


        if (CollectionUtils.isNotEmpty(list)) {
            page.setRecords(list);
        }
        return page;
    }

    @Override
    public Page<ExperiencePoolReportExportVo> viewAccountDetails(Long poolId, Long accountId, Integer state, Integer pageNo, Integer pageSize) {

        Page<ExperiencePoolReportExportVo> page = new Page<>(pageNo, pageSize);
        List<ExperiencePoolReportExportVo> list = studentExperienceMapper.viewAccountDetails(poolId, accountId, state, page);
        if (CollectionUtils.isNotEmpty(list)) {
            AccountVO accountVO = accountClient.findById(accountId);

            Integer startIndex = (pageNo - 1) * pageSize;
            Integer endIndex = Integer.MAX_VALUE;
            Long totalExperience = 0L;
            if (null == state) {
                totalExperience = studentExperienceMapper.getTotalExperience(poolId, accountId, startIndex, endIndex);
            }
            for (ExperiencePoolReportExportVo experiencePoolReportExportVo : list) {
                if (null != accountVO) {
                    experiencePoolReportExportVo.setAccountName(accountVO.getName());
                    experiencePoolReportExportVo.setAccountFullName(accountVO.getFullName() == null ? "" : accountVO.getFullName());
                    experiencePoolReportExportVo.setOrgName(accountVO.getOrgName() == null ? "" : accountVO.getOrgName());
                    experiencePoolReportExportVo.setOrgId(accountVO.getOrgId());
                    if (null == state) {
                        //剩余经验值
                        experiencePoolReportExportVo.setExperienceTotal(totalExperience);
                        //变化前前经验
                        totalExperience = totalExperience - experiencePoolReportExportVo.getExperience();
                    }
                    //变动明细
                    //课程
                    if (experiencePoolReportExportVo.getType() == 1) {
                        experiencePoolReportExportVo.setDescription("完成课程获得" + experiencePoolReportExportVo.getExperience() + "经验值");
                    }
                    if (experiencePoolReportExportVo.getType() == 2) {
                        experiencePoolReportExportVo.setDescription("完成考试获得" + experiencePoolReportExportVo.getExperience() + "经验值");
                    }
                    if (experiencePoolReportExportVo.getType() == 3) {
                        experiencePoolReportExportVo.setDescription("完成调研获得" + experiencePoolReportExportVo.getExperience() + "经验值");
                    }
//                    if (experiencePoolReportExportVo.getType()==4){
//                        experiencePoolReportExportVo.setDescription("完成投票获得"+experiencePoolReportExportVo.getExperience()+"经验值");
//                    }
                    if (experiencePoolReportExportVo.getType() == 5) {
                        experiencePoolReportExportVo.setDescription("外部导入获得" + experiencePoolReportExportVo.getExperience() + "经验值");
                    }
                    if (experiencePoolReportExportVo.getType() == 6) {
                        experiencePoolReportExportVo.setDescription("完成作业获得" + experiencePoolReportExportVo.getExperience() + "经验值");
                    }
                    if (experiencePoolReportExportVo.getType() == 10) {
                        experiencePoolReportExportVo.setDescription("购物使用" + (0 - experiencePoolReportExportVo.getExperience()) + "经验值");
                    }
                }
                page.setRecords(list);
            }
        }
        return page;
    }

    @Override
    public List<ExperiencePoolReportExportVo> viewAllDetails(Long poolId, String startTime, String endTime, String orgName, String accountName, Integer state, RequestContext res) {

        if (StringUtils.isBlank(startTime)) {
            startTime = null;
        }
        if (StringUtils.isBlank(endTime)) {
            endTime = null;
        }

        Integer visibleRange = 1;

        ExperiencePool experiencePool = poolMapper.selectById(poolId);
        if (null != experiencePool) {
            visibleRange = experiencePool.getVisibleRange();
        }

        List<ExperiencePoolReportExportVo> list = null;
        List<ReportAccountRespVO> accountVOList = null;
        List<ReportAccountRespVOVo> accountRespVOVos = new ArrayList<>();
        Map<Long, ReportAccountRespVO> accountVOMap = new HashMap<>();
        if (visibleRange == 1) {
            boolean isGetAccount = true;


            GetAccountListVO getAccountListVO = getAccountList(res, accountName, orgName, isGetAccount, accountVOList);

            if (null != getAccountListVO) {
                isGetAccount = getAccountListVO.getIsGetAccount();
                accountRespVOVos = getAccountListVO.getAccountVOList();
                if (!accountRespVOVos.isEmpty()) {
                    for (ReportAccountRespVOVo rvv : accountRespVOVos
                    ) {
                        ReportAccountRespVO rv = new ReportAccountRespVO();
                        BeanUtils.copyProperties(rvv, rv);
                        accountVOList.add(rv);
                    }
                }
            }

            List<Long> ids = new ArrayList<>();


            //如果全平台可见并且是管理员,先取出记录再根据记录取出对应用户信息,避免全平台人数过多,重复取出多余数据
            if (isGetAccount) {
                list = studentExperienceMapper.viewAllDetails(poolId, null, startTime, endTime, state);
                if (CollectionUtils.isNotEmpty(list)) {
                    for (ExperiencePoolReportExportVo vo : list) {
                        if (!ids.contains(vo.getAccountId())) {
                            ids.add(vo.getAccountId());
                        }
                    }
                    ReportRangeAccountReqVO reqVO = new ReportRangeAccountReqVO();
                    reqVO.setSiteId(res.getSiteId());
                    reqVO.setAccountIds(ids);
                    List<ReportAccountRespVO> respVOS = reportClient.getRangeAccounts(reqVO);
                    if (CollectionUtils.isNotEmpty(respVOS)) {
                        for (ReportAccountRespVO respVO : respVOS) {
                            accountVOMap.put(respVO.getUserId(), respVO);
                        }
                    }
                }
            } else {
                if (CollectionUtils.isNotEmpty(accountVOList)) {
                    for (ReportAccountRespVO reportAccountRespVO : accountVOList) {
                        ids.add(reportAccountRespVO.getUserId());
                        accountVOMap.put(reportAccountRespVO.getUserId(), reportAccountRespVO);
                    }
                    list = studentExperienceMapper.viewAllDetails(poolId, ids, startTime, endTime, state);
                } else {
                    return list;
                }

            }


        } else if (visibleRange == 2) {

            //可见范围用户id
            List<Long> authorizeIds = null;
            //可见范围内的管辖区用户
            List<Long> ids = new ArrayList<>();
            ExperiencePoolAuthorize poolAuthorize = new ExperiencePoolAuthorize();
            poolAuthorize.setExperienceId(poolId);
            EntityWrapper<ExperiencePoolAuthorize> entityWrapper = new EntityWrapper<>(poolAuthorize);
            List<ExperiencePoolAuthorize> authorizeList = poolAuthorizeMapper.selectList(entityWrapper);

            if (CollectionUtils.isNotEmpty(authorizeList)) {
                authorizeIds = authorizeList.stream().map(ExperiencePoolAuthorize::getRelationId).collect(Collectors.toList());
                LOGGER.info("#########################authorizeIds.size:" + authorizeIds.size());
            }

            if (CollectionUtils.isEmpty(authorizeIds)) {
                return null;
            }

            if (res.isAdmin()) {
                ReportRangeAccountReqVO reportRangeAccountReqVO = new ReportRangeAccountReqVO();
                reportRangeAccountReqVO.setSiteId(res.getSiteId());
                reportRangeAccountReqVO.setAccountIds(authorizeIds);
                if (StringUtils.isNotBlank(accountName)) {
                    reportRangeAccountReqVO.setAccountName(accountName.trim());
                }
                if (StringUtils.isNotBlank(orgName)) {
                    reportRangeAccountReqVO.setOrgName(orgName.trim());
                }
                accountVOList = reportClient.getRangeAccounts(reportRangeAccountReqVO);
                if (CollectionUtils.isNotEmpty(accountVOList)) {
                    for (ReportAccountRespVO reportAccountRespVO : accountVOList) {
                        if (!ids.contains(reportAccountRespVO.getUserId())) {
                            ids.add(reportAccountRespVO.getUserId());
                            accountVOMap.put(reportAccountRespVO.getUserId(), reportAccountRespVO);
                        }
                    }
                }
            } else {
                HQAccountInManageParam reportRangeAccountReqVO = new HQAccountInManageParam();
                reportRangeAccountReqVO.setAccountId(res.getAccountId());
                reportRangeAccountReqVO.setSiteId(res.getSiteId());
                reportRangeAccountReqVO.setModuleType(AuthzConstant.moduleType.statistics_experience);
                if (StringUtils.isNotBlank(accountName)) {
                    reportRangeAccountReqVO.setAccountName(accountName.trim());
                }
                if (StringUtils.isNotBlank(orgName)) {
                    reportRangeAccountReqVO.setOrgName(orgName.trim());
                }
                ReportAccountRespInManageVO reportAccountRespInManageVO = reportClient.getRangeAccountsInManage(reportRangeAccountReqVO);
                if (null != reportAccountRespInManageVO) {
                    if (!reportAccountRespInManageVO.getIsAdmin()) {
                        accountVOList = reportAccountRespInManageVO.getList();
                    } else {
                        ReportRangeAccountReqVO reportRangeAccountReqVO1 = new ReportRangeAccountReqVO();
                        reportRangeAccountReqVO1.setSiteId(res.getSiteId());
                        if (StringUtils.isNotBlank(accountName)) {
                            reportRangeAccountReqVO1.setAccountName(accountName.trim());
                        }
                        if (StringUtils.isNotBlank(orgName)) {
                            reportRangeAccountReqVO1.setOrgName(orgName.trim());
                        }
                        accountVOList = reportClient.getRangeAccounts(reportRangeAccountReqVO1);
                    }
                }

                if (CollectionUtils.isNotEmpty(accountVOList)) {
                    for (ReportAccountRespVO reportAccountRespVO : accountVOList) {
                        if (authorizeIds.contains(reportAccountRespVO.getUserId())) {
                            if (ids.contains(reportAccountRespVO.getUserId())) {
                                ids.add(reportAccountRespVO.getUserId());
                                accountVOMap.put(reportAccountRespVO.getUserId(), reportAccountRespVO);
                            }
                        }
                    }
                }
            }


            if (CollectionUtils.isNotEmpty(ids)) {
                list = studentExperienceMapper.viewAllDetails(poolId, ids, startTime, endTime, state);
                LOGGER.info("##########################所有人经验明细list.size:" + list.size());
            } else {
                return list;
            }
        }

        Map<Long, String> orgNameMap = new HashMap<>(16);
        //组装信息
        if (CollectionUtils.isNotEmpty(list)) {
            for (ExperiencePoolReportExportVo exportVo : list) {
                ReportAccountRespVO accountRespVO = accountVOMap.get(exportVo.getAccountId());
                if (null != accountRespVO) {
                    exportVo.setAccountName(accountRespVO.getUserName());
                    exportVo.setAccountFullName(accountRespVO.getUserFullName());
                    exportVo.setOrgName(accountRespVO.getOrgName());
                    exportVo.setOrgId(accountRespVO.getOrgId());
                    if (null != orgNameMap.get(accountRespVO)) {
                        exportVo.setOrgName(orgNameMap.get(accountRespVO));
                    } else {
                        String orgNames = "";
                        List<String> orgNamesList = organizationClient.getChiefOrgNames(accountRespVO.getOrgId(), accountRespVO.getCompanyId());
                        if (CollectionUtils.isNotEmpty(orgNamesList)) {
                            for (String s : orgNamesList) {
                                if (StringUtils.isNotBlank(orgNames)) {
                                    orgNames = orgNames + "/" + s;
                                } else {
                                    orgNames = orgNames + s;
                                }
                            }
                            exportVo.setOrgNames(orgNames);
                            orgNameMap.put(accountRespVO.getOrgId(), orgNames);
                        } else {
                            exportVo.setOrgNames("--");
                            orgNameMap.put(accountRespVO.getOrgId(), "--");
                        }
                    }
                    //变动明细
                    //课程
                    if (exportVo.getType() == 1) {
                        exportVo.setDescription("完成课程获得" + exportVo.getExperience() + "经验值");
                    }
                    if (exportVo.getType() == 2) {
                        exportVo.setDescription("完成考试获得" + exportVo.getExperience() + "经验值");
                    }
                    if (exportVo.getType() == 3) {
                        exportVo.setDescription("完成调研获得" + exportVo.getExperience() + "经验值");
                    }
//                    if (experiencePoolReportExportVo.getType()==4){
//                        experiencePoolReportExportVo.setDescription("完成投票获得"+experiencePoolReportExportVo.getExperience()+"经验值");
//                    }
                    if (exportVo.getType() == 5) {
                        exportVo.setDescription("外部导入获得" + exportVo.getExperience() + "经验值");
                    }
                    if (exportVo.getType() == 6) {
                        exportVo.setDescription("完成作业获得" + exportVo.getExperience() + "经验值");
                    }
                    if (exportVo.getType() == 10) {
                        exportVo.setDescription("兑换" + exportVo.getActivityName() + "消耗" + (0 - exportVo.getExperience()) + "经验值");
                    }
                }
            }
        }

        return list;
    }

    public GetAccountListVO getAccountList(RequestContext res, String key, String orgName, boolean isGetAccount, List<ReportAccountRespVO> accountVOList) {

        isGetAccount = true;

        if (res.isAdmin()) {

            if (StringUtils.isNotBlank(key) || StringUtils.isNotBlank(orgName)) {
                isGetAccount = false;
                ReportRangeAccountReqVO reportRangeAccountReqVO = new ReportRangeAccountReqVO();
                reportRangeAccountReqVO.setSiteId(res.getSiteId());
                if (StringUtils.isNotBlank(key)) {
                    reportRangeAccountReqVO.setAccountName(key.trim());
                }
                if (StringUtils.isNotBlank(orgName)) {
                    reportRangeAccountReqVO.setOrgName(orgName.trim());
                }
                accountVOList = reportClient.getRangeAccounts(reportRangeAccountReqVO);
            }

        } else {
            isGetAccount = false;
            HQAccountInManageParam reportRangeAccountReqVO = new HQAccountInManageParam();
            reportRangeAccountReqVO.setAccountId(res.getAccountId());
            reportRangeAccountReqVO.setSiteId(res.getSiteId());
            reportRangeAccountReqVO.setModuleType(AuthzConstant.moduleType.statistics_experience);
            if (StringUtils.isNotBlank(key)) {
                reportRangeAccountReqVO.setAccountName(key.trim());
            }
            ReportAccountRespInManageVO reportAccountRespInManageVO = reportClient.getRangeAccountsInManage(reportRangeAccountReqVO);
            if (null != reportAccountRespInManageVO) {
                if (!reportAccountRespInManageVO.getIsAdmin()) {
                    accountVOList = reportAccountRespInManageVO.getList();
                } else {
                    if (StringUtils.isNotBlank(key) || StringUtils.isNotBlank(orgName)) {
                        isGetAccount = false;
                        ReportRangeAccountReqVO reportRangeAccountReqVO1 = new ReportRangeAccountReqVO();
                        reportRangeAccountReqVO1.setSiteId(res.getSiteId());
                        if (StringUtils.isNotBlank(key)) {
                            reportRangeAccountReqVO1.setAccountName(key.trim());
                        }
                        if (StringUtils.isNotBlank(orgName)) {
                            reportRangeAccountReqVO1.setOrgName(orgName.trim());
                        }
                        accountVOList = reportClient.getRangeAccounts(reportRangeAccountReqVO1);
                    } else {
                        isGetAccount = true;
                    }
                }
            }
        }
        List<ReportAccountRespVOVo> accountRespVOVos=new ArrayList<>();
        if (CollectionUtils.isNotEmpty(accountVOList)) {
            for (ReportAccountRespVO rv : accountVOList
            ) {
                ReportAccountRespVOVo rvv = new ReportAccountRespVOVo();
                BeanUtils.copyProperties(rv, rvv);
                accountRespVOVos.add(rvv);
            }
        }
        GetAccountListVO getAccountListVO = new GetAccountListVO();
        getAccountListVO.setIsGetAccount(isGetAccount);
        getAccountListVO.setAccountVOList(accountRespVOVos);
        return getAccountListVO;
    }
}
