package com.yizhi.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.domain.ExperiencePoolProject;
import com.yizhi.application.domain.ProjectActivityExperience;
import com.yizhi.application.domain.ProjectActivityExperienceSet;
import com.yizhi.application.mapper.ProjectActivityExperienceMapper;
import com.yizhi.application.mapper.ProjectActivityExperienceSetMapper;
import com.yizhi.application.service.IProjectActivityExperienceService;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.custom.api.vo.custom.ExperienceRulesVO;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.application.orm.id.IdGenerator;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author ding
 * @since 2018-11-06
 */
@Service
public class ProjectActivityExperienceServiceImpl extends ServiceImpl<ProjectActivityExperienceMapper, ProjectActivityExperience> implements IProjectActivityExperienceService {

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ProjectActivityExperienceMapper activityExperienceMapper;

    @Autowired
    private ProjectActivityExperienceSetMapper activityExperienceSetMapper;

    @Override
    public List<ProjectActivityExperience> saveList(List<Long> deleteList, List<ExperiencePoolProject> poolProjectList) {

        List<ProjectActivityExperience> activityExperienceList = null;
        List<ProjectActivityExperience> insertList = null;
        RequestContext res = ContextHolder.get();
        if (CollectionUtils.isNotEmpty(poolProjectList)) {

            //删除已删除关联项目的活动经验
            if (CollectionUtils.isNotEmpty(deleteList)) {
                for (Long poolProjectId : deleteList) {
                    //如果新列表中删除,库里旧数据删除并删除对应活动经验数据
                    ProjectActivityExperience deleteActivityExperience = new ProjectActivityExperience();
                    deleteActivityExperience.setExperiencePoolProjectId(poolProjectId);
                    EntityWrapper<ProjectActivityExperience> deleteWrapper = new EntityWrapper<>(deleteActivityExperience);
                    this.delete(deleteWrapper);
                }
            }
        }

        insertList = new ArrayList<>();
        activityExperienceList = new ArrayList<>();
        for (ExperiencePoolProject poolProject : poolProjectList) {
            ProjectActivityExperience activityExperience = new ProjectActivityExperience();

            //查找是否有记录,没有就新建入库
            activityExperience.setExperiencePoolProjectId(poolProject.getId());
            EntityWrapper<ProjectActivityExperience> entityWrapper = new EntityWrapper<>(activityExperience);
            List<ProjectActivityExperience> list = this.selectList(entityWrapper);
            if (CollectionUtils.isNotEmpty(list)) {
                activityExperienceList.addAll(list);
            } else {
                activityExperience.setId(idGenerator.generate());
                activityExperience.setType(poolProject.getType());
                activityExperience.setActivityName(poolProject.getBizName());
                activityExperience.setActivityExperienceSource(0);
                activityExperience.setCreateById(res.getAccountId());
                activityExperience.setCreateByName(res.getAccountFullName());
                activityExperience.setCreateTime(new Date());
                insertList.add(activityExperience);
                activityExperienceList.add(activityExperience);
            }
        }
        //新建的入库
        if (CollectionUtils.isNotEmpty(insertList)) {
            this.insertBatch(insertList);
        }
        return activityExperienceList;
    }

    @Override
    public boolean insertList(List<ProjectActivityExperience> projectActivityExperienceList) {
        RequestContext res = ContextHolder.get();
        List<ProjectActivityExperience> updateList = new ArrayList<>();
        for (ProjectActivityExperience activityExperience : projectActivityExperienceList) {
            if (activityExperience.getType() == 2) {
                ProjectActivityExperience activityExperience1 = new ProjectActivityExperience();
                activityExperience1 = this.selectById(activityExperience.getId());
                activityExperience.setUpdateById(res.getAccountId());
                activityExperience.setUpdateByName(res.getAccountFullName());
                activityExperience.setUpdateTime(new Date());
                if (activityExperience1.getActivityExperienceSource() == 0 && null == activityExperience.getActivityExperience()) {
                    activityExperience.setActivityExperience(0L);
                } else if (activityExperience1.getActivityExperienceSource() == 1) {
                    activityExperience.setActivityExperienceSource(1);
                }
                activityExperience.setEvaluate(1);
            } else {
                activityExperience.setUpdateById(res.getAccountId());
                activityExperience.setUpdateByName(res.getAccountFullName());
                activityExperience.setUpdateTime(new Date());
                if (null == activityExperience.getActivityExperience()) {
                    activityExperience.setActivityExperience(0L);
                }
                activityExperience.setEvaluate(1);

            }

            updateList.add(activityExperience);
        }
        return this.updateBatchById(updateList);
    }

    @Override
    public List<ExperienceRulesVO> getExperienceRules(Long poolId) {
        List<ExperienceRulesVO> resultList = new ArrayList<>();
        ExperienceRulesVO experienceRulesVO = null;
        List<ProjectActivityExperience> activityExperienceList = activityExperienceMapper.getActivityExperienceList(poolId);
        if (CollectionUtils.isNotEmpty(activityExperienceList)) {
            for (ProjectActivityExperience projectActivityExperience : activityExperienceList) {
                //判断是否是完成即获取,如果是直接拿出数据组装
                if (projectActivityExperience.getActivityExperienceSource() == 0) {
                    experienceRulesVO = new ExperienceRulesVO();
                    experienceRulesVO.setType(projectActivityExperience.getType());
                    experienceRulesVO.setName(projectActivityExperience.getActivityName());
                    experienceRulesVO.setExperienceSource(1);
                    experienceRulesVO.setExperience(null == projectActivityExperience.getActivityExperience() ? "0" : projectActivityExperience.getActivityExperience().toString());
                    resultList.add(experienceRulesVO);
                } else {
                    //如果不是,查找规则设置再分别组装
                    List<ProjectActivityExperienceSet> activityExperienceSetList = new ArrayList<>();
                    ProjectActivityExperienceSet activityExperienceSet = new ProjectActivityExperienceSet();
                    activityExperienceSet.setProjectActivityExperienceId(projectActivityExperience.getId());
                    EntityWrapper<ProjectActivityExperienceSet> entityWrapper = new EntityWrapper<>(activityExperienceSet);
                    activityExperienceSetList = activityExperienceSetMapper.selectList(entityWrapper);
                    if (CollectionUtils.isNotEmpty(activityExperienceSetList)) {
                        for (ProjectActivityExperienceSet projectActivityExperienceSet : activityExperienceSetList) {
                            experienceRulesVO = new ExperienceRulesVO();
                            if (projectActivityExperienceSet.getSetType() == 1) {
                                experienceRulesVO.setType(projectActivityExperience.getType());
                                experienceRulesVO.setName(projectActivityExperience.getActivityName());
                                experienceRulesVO.setExperienceSource(2);
                                experienceRulesVO.setMinScore(projectActivityExperienceSet.getMinScore());
                                Integer maxScore = activityExperienceSetMapper.getMaxScore(projectActivityExperienceSet.getProjectActivityExperienceId(), projectActivityExperienceSet.getMinScore());
                                experienceRulesVO.setMaxScore(maxScore);
                                experienceRulesVO.setExperience(projectActivityExperienceSet.getExperience().toString());
                                resultList.add(experienceRulesVO);
                            } else {
                                experienceRulesVO.setType(projectActivityExperience.getType());
                                experienceRulesVO.setName(projectActivityExperience.getActivityName());
                                experienceRulesVO.setExperienceSource(3);
                                experienceRulesVO.setExperience("成绩" + "*" + projectActivityExperienceSet.getTimes());
                                resultList.add(experienceRulesVO);
                            }
                        }
                    }
                }
            }
        }
        return resultList;
    }

}
