package com.yizhi.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;

import com.yizhi.assignment.application.feign.AssignmentClient;
import com.yizhi.course.application.feign.RecordeClient;
import com.yizhi.application.constant.PoolConstant;
import com.yizhi.application.domain.*;
import com.yizhi.application.listener.ExperienceListener;
import com.yizhi.application.mapper.ExperiencePoolProjectMapper;
import com.yizhi.application.mapper.ProjectActivityExperienceSetMapper;
import com.yizhi.application.mapper.StudentExperienceMapper;
import com.yizhi.application.service.IExperiencePoolAuthorizeService;
import com.yizhi.application.service.IExperiencePoolService;
import com.yizhi.application.service.IProjectActivityExperienceSetService;
import com.yizhi.application.service.IStudentExperienceService;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.custom.api.vo.custom.ExamRuleListVo;
import com.yizhi.custom.api.vo.custom.ExperienceOrgRankingVO;
import com.yizhi.custom.api.vo.custom.ExperiencePoolExamRuleVo;
import com.yizhi.custom.api.vo.custom.ExperiencePoolRuleVo;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.exam.application.feign.ExamAnswerClient;
import com.yizhi.point.application.feign.PointActivityFeignClients;
import com.yizhi.point.application.vo.PointParamVO;
import com.yizhi.system.application.system.remote.OrganizationClient;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.vo.ReportAccountRespVO;
import com.yizhi.system.application.vo.ReportRangeAccountReqVO;
import com.yizhi.util.application.constant.QueueConstant;
import com.yizhi.util.application.event.TrainingProjectEvent;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 项目关联活动经验值表 服务实现类
 * </p>
 *
 * @author dingxiaowei123
 * @since 2018-11-12
 */
@Service
public class StudentExperienceServiceImpl extends ServiceImpl<StudentExperienceMapper, StudentExperience> implements IStudentExperienceService {

    private static final Logger LOGGER = LoggerFactory.getLogger(StudentExperienceServiceImpl.class);

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private StudentExperienceMapper studentExperienceMapper;

    @Autowired
    private IExperiencePoolService poolService;

    @Autowired
    private IProjectActivityExperienceSetService experienceSetService;

    @Autowired
    private ProjectActivityExperienceSetMapper experienceSetMapper;

    @Autowired
    private ExperiencePoolProjectMapper experiencePoolProjectMapper;

    @Autowired
    private ExperienceListener experienceListener;

    @Autowired
    private PointActivityFeignClients pointActivityFeignClients;

    @Autowired
    private RecordeClient recordeClient;

    @Autowired
    private ExamAnswerClient examAnswerClient;

    @Autowired
    private IExperiencePoolAuthorizeService experiencePoolAuthorizeService;

    @Autowired
    private ReportClient reportClient;

    @Autowired
    private OrganizationClient organizationClient;

    @Autowired
    private AssignmentClient assignmentClient;

    @Autowired
    private TaskExecutor taskExecutor;

    @Autowired
    private CloudEventPublisher cloudEventPublisher;

    @Override
    public boolean insertStudentExperienceList(List<StudentExperience> studentExperienceList) {
        if (CollectionUtils.isNotEmpty(studentExperienceList)) {
            for (StudentExperience studentExperience : studentExperienceList) {
                //获取id
                studentExperience.setId(idGenerator.generate());
                studentExperience.setCreateTime(new Date());
            }
        }
        LOGGER.info("----------", studentExperienceList.size());
        return this.insertOrUpdateBatch(studentExperienceList);
    }

    @Override
    public List<ExperiencePoolRuleVo> getRules(Long activityId, Long compantyId, Long siteId, Long accountId) {
        List<ExperiencePoolRuleVo> experiencePoolRuleVos = studentExperienceMapper.getRules(activityId, compantyId, siteId);
        if (CollectionUtils.isNotEmpty(experiencePoolRuleVos)) {
            for (ExperiencePoolRuleVo experiencePoolRuleVo : experiencePoolRuleVos) {
                if (experiencePoolRuleVo.getVisibleRange() == 1) {
                    continue;
                } else {
                    //如果是指定范围,验证指定范围
                    ExperiencePoolAuthorize experiencePoolAuthorize = new ExperiencePoolAuthorize();
                    experiencePoolAuthorize.setExperienceId(experiencePoolRuleVo.getExperiencePoolId());
                    List<ExperiencePoolAuthorize> authorizeList = experiencePoolAuthorizeService.selectList(new EntityWrapper<>(experiencePoolAuthorize));
                    if (CollectionUtils.isNotEmpty(authorizeList)) {
                        List<Long> accountIds = authorizeList.stream().map(ExperiencePoolAuthorize::getRelationId).collect(Collectors.toList());
                        if (null != accountId) {
                            //如果实时发放
                            if (!accountIds.contains(accountId)) {
                                continue;
                            }
                        } else {
                            //如果补发
                            experiencePoolRuleVo.setAccountIds(accountIds);
                        }
                    } else {
                        continue;
                    }
                }
            }
        }
        return experiencePoolRuleVos;
    }

    @Override
    public void insertStudentExperience(StudentExperience studentExperience) {
        if (null != studentExperience) {
            studentExperience.setId(idGenerator.generate());
            this.insert(studentExperience);
        }
    }

    @Override
    public List<ExperiencePoolExamRuleVo> getExamRules(Long activityId, Long compantyId, Long siteId, Long accountId, Integer type) {
        List<ExperiencePoolExamRuleVo> list = new ArrayList<>();
        ExperiencePool experiencePool = new ExperiencePool();
        experiencePool.setCompanyId(compantyId);
        experiencePool.setSiteId(siteId);
        experiencePool.setState(2);
        EntityWrapper<ExperiencePool> entityWrapper = new EntityWrapper<>(experiencePool);
        List<ExperiencePool> poolList = poolService.selectList(entityWrapper);
        if (CollectionUtils.isNotEmpty(poolList)) {
            for (ExperiencePool experiencePool1 : poolList) {
                ExperiencePoolExamRuleVo examRuleVo = new ExperiencePoolExamRuleVo();
                //如果是指定范围,验证指定范围
                if (experiencePool1.getVisibleRange() == 2) {
                    ExperiencePoolAuthorize experiencePoolAuthorize = new ExperiencePoolAuthorize();
                    experiencePoolAuthorize.setExperienceId(experiencePool1.getId());
                    List<ExperiencePoolAuthorize> authorizeList = experiencePoolAuthorizeService.selectList(new EntityWrapper<>(experiencePoolAuthorize));
                    if (CollectionUtils.isNotEmpty(authorizeList)) {
                        List accountIds = authorizeList.stream().map(ExperiencePoolAuthorize::getRelationId).collect(Collectors.toList());
                        if (accountId == null) {
                            //补发验证实时发放验证
                            examRuleVo.setAccountIds(accountIds);
                        } else {
                            //实时发放验证
                            if (!accountIds.contains(accountId)) {
                                continue;
                            }
                        }
                    } else {
                        continue;
                    }
                }
                ProjectActivityExperience activityExperience = studentExperienceMapper.getExamRules(experiencePool1.getId(), activityId);
                if (null == activityExperience) {
                    continue;
                }
                examRuleVo.setExperiencePoolId(experiencePool1.getId());
                examRuleVo.setBizId(activityId);
                examRuleVo.setBizName(activityExperience.getActivityName());
                examRuleVo.setType(type);
                //如果无其他条件
                if (activityExperience.getActivityExperienceSource() == 0) {
                    examRuleVo.setExperienceType(1);
                    ExamRuleListVo examRuleListVo = new ExamRuleListVo();
                    examRuleListVo.setExperience(activityExperience.getActivityExperience());
                    List<ExamRuleListVo> examRuleListVoList = new ArrayList<>();
                    examRuleListVoList.add(examRuleListVo);
                    examRuleVo.setRuleList(examRuleListVoList);
                    list.add(examRuleVo);
                } else {
                    ProjectActivityExperienceSet experienceSet = new ProjectActivityExperienceSet();
                    experienceSet.setProjectActivityExperienceId(activityExperience.getId());
                    EntityWrapper<ProjectActivityExperienceSet> entityWrapper1 = new EntityWrapper<>(experienceSet);
                    entityWrapper1.orderBy("min_score", true);
                    List<ProjectActivityExperienceSet> experienceSetList = experienceSetService.selectList(entityWrapper1);
                    if (CollectionUtils.isNotEmpty(experienceSetList)) {
                        //按考试成绩范围
                        if (experienceSetList.get(0).getSetType() == 1) {
                            List<ExamRuleListVo> examRuleListVoList = new ArrayList<>();
                            for (ProjectActivityExperienceSet experienceSet1 : experienceSetList) {
                                Integer minScore = experienceSet1.getMinScore();
                                Integer maxScore = experienceSetMapper.getMaxScore(experienceSet1.getProjectActivityExperienceId(), minScore);
                                ExamRuleListVo examRuleListVo = new ExamRuleListVo();
                                examRuleListVo.setMinScore(minScore);
                                examRuleListVo.setExperience(experienceSet1.getExperience());
                                if (null != maxScore) {
                                    examRuleListVo.setMaxScore(maxScore);
                                }
                                examRuleListVoList.add(examRuleListVo);
                            }
                            examRuleVo.setExperienceType(2);
                            examRuleVo.setRuleList(examRuleListVoList);
                        } else {
                            //按考试成绩倍数
                            List<ExamRuleListVo> examRuleListVoList = new ArrayList<>();
                            ExamRuleListVo examRuleListVo = new ExamRuleListVo();
                            examRuleListVo.setTimes(experienceSetList.get(0).getTimes());
                            examRuleListVoList.add(examRuleListVo);
                            examRuleVo.setExperienceType(3);
                            examRuleVo.setRuleList(examRuleListVoList);
                        }
                    }
                    list.add(examRuleVo);
                }
            }
        }
        return list;
    }

    /**
     * 经验池手动补发经验
     *
     * @param experiencePoolId
     * @return
     */
    @Override
    public boolean pushStudentExperience(Long experiencePoolId, Long activityId, String activityName, Integer activityType, List<Long> accountIds, RequestContext requestContext) {
        ExperiencePoolProject experiencePoolProject = new ExperiencePoolProject();
        Long companyId = requestContext.getCompanyId();
        Long siteId = requestContext.getSiteId();

        //获取用户信息并放入map中
        ReportRangeAccountReqVO reqVO = new ReportRangeAccountReqVO();
        reqVO.setSiteId(siteId);
        reqVO.setAccountIds(accountIds);
        List<ReportAccountRespVO> accounts = reportClient.getRangeAccounts(reqVO);
        Map<Long, ReportAccountRespVO> map = new HashMap<>(16);
        if (CollectionUtils.isNotEmpty(accounts)) {
            for (ReportAccountRespVO account : accounts) {
                map.put(account.getUserId(), account);
            }
        }

        if (activityType.equals(PoolConstant.EXAMTYPE) || activityType.equals(PoolConstant.ASSIGNMENTTYPE)) {
            //查找经验发放规则
            List<ExperiencePoolExamRuleVo> examRuleVos = this.getExamRules(activityId, companyId, siteId, null, activityType);
            if (CollectionUtils.isNotEmpty(examRuleVos)) {
                for (ExperiencePoolExamRuleVo examRuleVo : examRuleVos) {
                    //如果找到对应经验池相关经验发放规则
                    if (examRuleVo.getExperiencePoolId().equals(experiencePoolId)) {

                        for (Long accountId : accountIds) {
                            if (CollectionUtils.isNotEmpty(examRuleVo.getAccountIds())) {
                                if (!examRuleVo.getAccountIds().contains(accountId)) {
                                    continue;
                                }
                            }
                            PointParamVO vo = new PointParamVO();
                            ReportAccountRespVO accountRespVO = map.get(accountId);
                            vo.setAccountId(accountId);
                            vo.setCompanyId(companyId);
                            if (null != accountRespVO) {
                                vo.setOrgId(accountRespVO.getOrgId());
                            } else {
                                continue;
                            }
                            vo.setSiteId(siteId);
                            vo.setActivityName(activityName);
                            if (examRuleVo.getExperienceType() == 1) {
                                ExperiencePoolRuleVo experiencePoolRuleVo = new ExperiencePoolRuleVo();
                                experiencePoolRuleVo.setExperiencePoolId(examRuleVo.getExperiencePoolId());
                                experiencePoolRuleVo.setType(examRuleVo.getType());
                                experiencePoolRuleVo.setBizId(examRuleVo.getBizId());
                                experiencePoolRuleVo.setBizName(examRuleVo.getBizName());
                                experiencePoolRuleVo.setExperience(examRuleVo.getRuleList().get(0).getExperience());
                                boolean success = experienceListener.addExperience(experiencePoolRuleVo, vo, activityType);
                                if (success) {
                                    LOGGER.info("考试{}中学员{}经验补发成功", activityId, accountId);
                                } else {
                                    LOGGER.info("考试{}中学员{}经验补发失败", activityId, accountId);
                                }
                            } else {
                                Integer score = null;
                                if (activityType.equals(PoolConstant.EXAMTYPE)) {
                                    score = examAnswerClient.getMaxScore(activityId, accountId, companyId, siteId);
                                } else {
                                    score = assignmentClient.getScore(activityId, siteId, companyId, accountId);
                                }
                                if (null == score) {
                                    LOGGER.info("未找到考试{}中学员{}的考试分数", accountId, accountId);
                                    continue;
                                }
                                vo.setScore(score);
                                boolean success = experienceListener.addScoreExperience(examRuleVo, vo, activityType);
                                if (success) {
                                    LOGGER.info("考试{}中学员{}经验补发成功", activityId, accountId);
                                } else {
                                    LOGGER.info("考试{}中学员{}经验补发失败", activityId, accountId);
                                }
                            }
                        }
                    }
                }
                return true;
            }
        } else {
            List<ExperiencePoolRuleVo> ruleVoList = this.getRules(activityId, companyId, siteId, null);
            if (CollectionUtils.isNotEmpty(ruleVoList)) {
                for (ExperiencePoolRuleVo ruleVo : ruleVoList) {
                    //如果找到相关经验发放规则
                    if (ruleVo.getExperiencePoolId().equals(experiencePoolId)) {
                        if (activityType.equals(PoolConstant.COURSETYPE)) {
                            for (Long accountId : accountIds) {
                                //如果是指定范围,验证是否在指定范围
                                if (CollectionUtils.isNotEmpty(ruleVo.getAccountIds())) {
                                    if (!ruleVo.getAccountIds().contains(accountId)) {
                                        continue;
                                    }
                                }
                                PointParamVO vo = new PointParamVO();
                                ReportAccountRespVO accountRespVO = map.get(accountId);
                                vo.setAccountId(accountId);
                                vo.setCompanyId(companyId);
                                if (null != accountRespVO) {
                                    vo.setOrgId(accountRespVO.getOrgId());
                                } else {
                                    continue;
                                }
                                vo.setSiteId(siteId);
                                vo.setActivityName(activityName);

                                Date time = new Date();
                                boolean finalIsFinished = true;

                                taskExecutor.asynExecute(new AbstractTaskHandler() {
                                    @Override
                                    public void handle() {
                                        try {
                                            cloudEventPublisher.publish(QueueConstant.TRAINING_PROJECT_EVENT_QUEUE,
                                                    new EventWrapper<TrainingProjectEvent>(ruleVo.getBizId(),
                                                            TrainingProjectEvent.getInstanceCourse(ruleVo.getBizId(), accountId, time, 0L,
                                                                    finalIsFinished, requestContext.getSiteId(), 1)));
                                        } catch (Exception e) {
                                            e.printStackTrace();
                                            LOGGER.error("发送课程:(" + ruleVo.getBizName() + ") 完成信息到trainingProject异常：", e);
                                        }
                                    }
                                });

                                boolean success = experienceListener.addExperience(ruleVo, vo, PoolConstant.COURSETYPE);
                                if (success) {
                                    LOGGER.info("课程{}中学员{}经验补发成功", activityId, accountId);
                                } else {
                                    LOGGER.info("课程{}中学员{}经验补发失败", activityId, accountId);
                                }
                            }
                        } else if (activityType.equals(PoolConstant.RESEARCHTYPE)) {
                            for (Long accountId : accountIds) {
                                //如果是指定范围,验证是否在指定范围
                                if (CollectionUtils.isNotEmpty(ruleVo.getAccountIds())) {
                                    if (!ruleVo.getAccountIds().contains(accountId)) {
                                        continue;
                                    }
                                }
                                PointParamVO vo = new PointParamVO();
                                ReportAccountRespVO accountRespVO = map.get(accountId);
                                vo.setAccountId(accountId);
                                vo.setCompanyId(companyId);
                                if (null != accountRespVO) {
                                    vo.setOrgId(accountRespVO.getOrgId());
                                } else {
                                    continue;
                                }
                                vo.setSiteId(siteId);
                                vo.setActivityName(activityName);
                                boolean success = experienceListener.addExperience(ruleVo, vo, PoolConstant.RESEARCHTYPE);
                                if (success) {
                                    LOGGER.info("调研{}中学员{}经验补发成功", activityId, accountId);
                                } else {
                                    LOGGER.info("调研{}中学员{}经验补发失败", activityId, accountId);
                                }
                            }
                        } else if (activityType.equals(PoolConstant.VOTETYPE)) {
                            for (Long accountId : accountIds) {
                                //如果是指定范围,验证是否在指定范围
                                if (CollectionUtils.isNotEmpty(ruleVo.getAccountIds())) {
                                    if (!ruleVo.getAccountIds().contains(accountId)) {
                                        continue;
                                    }
                                }
                                PointParamVO vo = new PointParamVO();
                                ReportAccountRespVO accountRespVO = map.get(accountId);
                                vo.setAccountId(accountId);
                                vo.setCompanyId(companyId);
                                if (null != accountRespVO) {
                                    vo.setOrgId(accountRespVO.getOrgId());
                                } else {
                                    continue;
                                }
                                vo.setSiteId(siteId);
                                vo.setActivityName(activityName);
                                boolean success = experienceListener.addExperience(ruleVo, vo, PoolConstant.VOTETYPE);
                                if (success) {
                                    LOGGER.info("投票{}中学员{}经验补发成功", activityId, accountId);
                                } else {
                                    LOGGER.info("投票{}中学员{}经验补发失败", activityId, accountId);
                                }
                            }
                        }
                    }
                }
                return true;
            }
        }
        return false;
    }

    @Override
    public Page<ExperienceOrgRankingVO> getOrgRanking(Long poolId, Integer pageNo, Integer pageSize, Integer type) {
        Page<ExperienceOrgRankingVO> page = new Page<>(pageNo, pageSize);
        if (type == 0) {
            List<ExperienceOrgRankingVO> list = studentExperienceMapper.getOrgRankings(poolId, page);
            if (CollectionUtils.isNotEmpty(list)) {
                List<Long> orgIds = list.stream().map(ExperienceOrgRankingVO::getOrgId).collect(Collectors.toList());

                Map<Long, String> map = organizationClient.getOrgNames(orgIds);
                for (ExperienceOrgRankingVO rankingVO : list) {
                    if (null != map) {
                        rankingVO.setOrgName(map.get(rankingVO.getOrgId()));
                    }
                    Integer total = studentExperienceMapper.getOrgRanking(poolId, rankingVO.getTotalExperience());
                    if (null == total) {
                        total = 0;
                    } else {
                        total += 1;
                    }
                    rankingVO.setRankings(total);
                }
                page.setRecords(list);
            }
        } else if (type == 1) {
            //获得按组别排名(平均经验)
            List<ExperienceOrgRankingVO> list = studentExperienceMapper.getGroupRankings(poolId, page);
            //获得名次(平均经验)
            if (CollectionUtils.isNotEmpty(list)) {
                for (ExperienceOrgRankingVO experienceOrgRankingVO : list) {
                    Integer total = studentExperienceMapper.getGroupRanking(poolId, experienceOrgRankingVO.getTotalExperience());
                    if (null == total) {
                        total = 0;
                    } else {
                        total += 1;
                    }
                    experienceOrgRankingVO.setRankings(total);
                }
                page.setRecords(list);
            }
        } else if (type == 2) {
            //获得按组别排名,可导入外部经验(平均经验,每个部门新建一个人专门导入经验,算平均分的时候不算在内)
            List<ExperienceOrgRankingVO> list = studentExperienceMapper.getGroupRankingsWithImport(poolId, page);
            //获得名次(平均经验)
            if (CollectionUtils.isNotEmpty(list)) {
                for (ExperienceOrgRankingVO experienceOrgRankingVO : list) {
                    Integer total = studentExperienceMapper.getGroupRanking(poolId, experienceOrgRankingVO.getTotalExperience());
                    if (null == total) {
                        total = 0;
                    } else {
                        total += 1;
                    }
                    experienceOrgRankingVO.setRankings(total);
                }
                page.setRecords(list);
            }
        }

        return page;
    }

    @Override
    public Long getTotalExperience(Long poolId, Long accountId) {
        return studentExperienceMapper.getAccountTotalExperience(poolId, accountId);
    }
}
